from ct.crypto import error
from ct.crypto import pem
from ct.crypto.asn1 import types
from ct.proto import client_pb2

import hashlib
import ecdsa

class _ECDSASignature(types.Sequence):
    components = (
            (types.Component("r", types.Integer)),
            (types.Component("s", types.Integer))
    )

class EcdsaVerifier(object):
    """Verifies ECDSA signatures."""

    # The signature algorithm used for this public key."""
    SIGNATURE_ALGORITHM = client_pb2.DigitallySigned.ECDSA
    # The hash algorithm used for this public key."""
    HASH_ALGORITHM = client_pb2.DigitallySigned.SHA256

    # Markers to look for when reading a PEM-encoded ECDSA public key."""
    __READ_MARKERS = ("PUBLIC KEY", "ECDSA PUBLIC KEY")
    # A marker to write when writing a PEM-encoded ECDSA public key."""
    __WRITE_MARKER = "ECDSA PUBLIC KEY"

    def __init__(self, key_info):
        """Creates a verifier that uses a PEM-encoded ECDSA public key.

        Args:
        - key_info: KeyInfo protobuf message

        Raises:
        - PemError: If the key has an invalid encoding
        """
        if (key_info.type != client_pb2.KeyInfo.ECDSA):
            raise error.UnsupportedAlgorithmError(
                "Expected ECDSA key, but got key type %d" % key_info.type)

        # Will raise a PemError on invalid encoding
        self.__der, _ = pem.from_pem(key_info.pem_key, self.__READ_MARKERS)
        try:
            self.__key = ecdsa.VerifyingKey.from_der(self.__der)
        except ecdsa.der.UnexpectedDER as e:
            raise error.EncodingError(e)

    def __repr__(self):
        return "%s(public key: %r)" % (self.__class__.__name__,
                                       pem.to_pem(self.__der,
                                                  self.__WRITE_MARKER))

    @error.returns_true_or_raises
    def verify(self, signature_input, signature):
        """Verifies the signature was created by the owner of the public key.

        Args:
        - signature_input: The data that was originally signed.
        - signature: An ECDSA SHA256 signature.

        Returns:
        - True if the signature verifies.

        Raises:
        - error.EncodingError: If the signature encoding is invalid.
        - error.SignatureError: If the signature fails verification.
        """
        try:
            _ECDSASignature.decode(signature)
            return self.__key.verify(signature, signature_input,
                                     hashfunc=hashlib.sha256,
                                     sigdecode=ecdsa.util.sigdecode_der)
        except (ecdsa.der.UnexpectedDER, error.ASN1Error) as e:
            raise error.EncodingError("Invalid DER encoding for signature %s",
                                      signature.encode("hex"), e)
        except ecdsa.keys.BadSignatureError:
            raise error.SignatureError("Signature did not verify: %s",
                                       signature.encode("hex"))




#!/usr/bin/env python

import os
import sys

import gflags

from ct.client import scanner

FLAGS = gflags.FLAGS

gflags.DEFINE_integer("multi", 2, "Number of cert parsing processes to use in "
                      "addition to the main process and the network process.")
gflags.DEFINE_string("output", None,
                     "Output directory to write certificates to.")


def match(certificate, entry_type, extra_data, certificate_index):
    # Fill this in with your match criteria, e.g.
    #
    # return "google" in certificate.subject_name().lower()
    #
    # NB: for precertificates, issuer matching may not work as expected
    # when the precertificate has been issued by the special-purpose
    # precertificate signing certificate.
    return ("cert_%d.der" % certificate_index, certificate.to_der())

def write_matched_certificate(matcher_output):
    output_file, der_data = matcher_output
    with open(os.path.join(FLAGS.output, output_file), "wb") as f:
        f.write(der_data)


def run():
    if not FLAGS.output:
        raise Exception("Certificates output directory must be specified.")

    res = scanner.scan_log(
        match, "https://ct.googleapis.com/pilot", FLAGS.multi,
        write_matched_certificate)
    print "Scanned %d, %d matched and %d failed strict or partial parsing" % (
        res.total, res.matches, res.errors)


if __name__ == "__main__":
    sys.argv = FLAGS(sys.argv)
    run()



#!/usr/bin/env python
"""Submits a chain to a list of logs."""

import base64
import hashlib
import sys

import json
import logging
import gflags

from ct.client import log_client
from ct.crypto import cert
from ct.crypto import error
from ct.crypto import pem
from ct.crypto import verify
from ct.proto import client_pb2
from ct.serialization import tls_message

FLAGS = gflags.FLAGS

gflags.DEFINE_string("log_list", None, "File containing the list of logs "
                     "to submit to (see certificate-transparency.org/known-logs"
                     " for the format description).")
gflags.DEFINE_string("chain", None, "Certificate chain to submit (PEM).")
gflags.DEFINE_string("log_scheme", "http", "Log scheme (http/https)")
gflags.DEFINE_string("output", None, "output file for sct_list")
gflags.MarkFlagAsRequired("log_list")
gflags.MarkFlagAsRequired("chain")
gflags.MarkFlagAsRequired("output")

def _read_ct_log_list(log_list_file):
    """Parses the log list JSON, returns a log url to key map."""
    try:
        log_list_json = json.loads(log_list_file)
        log_url_to_key = {}
        for log_info in log_list_json['logs']:
            log_url_to_key[FLAGS.log_scheme + '://' + log_info['url']] = (
                base64.decodestring(log_info['key']))
        return log_url_to_key
    except (OSError, IOError) as io_exception:
        raise Exception('Could not read log list file %s: %s' %
                        (log_list_file, io_exception))

def _submit_to_single_log(log_url, full_chain):
    """Submits the chain to a single log specified by log_url."""
    ct_client = log_client.LogClient(log_url, connection_timeout=10)
    res = None
    try:
        res = ct_client.add_chain(full_chain)
    except log_client.HTTPError as err:
        logging.info('Skipping log %s because of error: %s\n', log_url, err)
    return res

def _map_log_id_to_verifier(log_list):
    """Returns a map from log id to verifier object from the log_list."""
    log_id_to_verifier = {}
    for log_key in log_list.values():
        key_info = verify.create_key_info_from_raw_key(log_key)
        key_id = hashlib.sha256(log_key).digest()
        log_id_to_verifier[key_id] = verify.LogVerifier(key_info)
    return log_id_to_verifier

def _submit_to_all_logs(log_list, certs_chain):
    """Submits the chain to all logs in log_list and validates SCTs."""
    log_id_to_verifier = _map_log_id_to_verifier(log_list)

    chain_der = [c.to_der() for c in certs_chain]
    raw_scts_for_cert = []
    for log_url in log_list.keys():
        res = _submit_to_single_log(log_url, chain_der)
        if res:
            raw_scts_for_cert.append(res)
        else:
            logging.info("No SCT from log %s", log_url)

    validated_scts = []
    for raw_sct in raw_scts_for_cert:
        key_id = raw_sct.id.key_id
        try:
            log_id_to_verifier[key_id].verify_sct(raw_sct, certs_chain)
            validated_scts.append(raw_sct)
        except error.SignatureError as err:
            logging.warning(
                    'Discarding SCT from log_id %s which does not validate: %s',
                    key_id.encode('hex'), err)
        except KeyError as err:
            logging.warning('Could not find CT log validator for log_id %s. '
                    'The log key for this log is probably misconfigured.',
                    key_id.encode('hex'))

    scts_for_cert = [tls_message.encode(proto_sct)
            for proto_sct in validated_scts
            if proto_sct]
    sct_list = client_pb2.SignedCertificateTimestampList()
    sct_list.sct_list.extend(scts_for_cert)
    return tls_message.encode(sct_list)

def run():
    """Submits the chain specified in the flags to all logs."""
    logging.getLogger().setLevel(logging.INFO)
    logging.info("Starting up.")
    with open(FLAGS.log_list) as log_list_file:
        log_url_to_key = _read_ct_log_list(log_list_file.read())

    certs_chain = [c for c in cert.certs_from_pem_file(FLAGS.chain)]
    logging.info("Chain is of length %d", len(certs_chain))

    sct_list = _submit_to_all_logs(log_url_to_key, certs_chain)
    with open(FLAGS.output, 'wb') as sct_list_file:
        sct_list_file.write(sct_list)

if __name__ == "__main__":
    sys.argv = FLAGS(sys.argv)
    run()



import os

from google.protobuf import message

class Error(Exception):
    pass

class WriteError(Error):
    pass

class ReadError(Error):
    pass

class FileNotFoundError(ReadError):
    pass

class CorruptStateError(ReadError):
    pass

class StateKeeper(object):
    def __init__(self, state_file):
        self.__state_file = state_file

    def write(self, state):
        """Write a protocol buffer to the state file."""
        serialized_state = state.SerializeToString()
        try:
            with open(self.__state_file, "wb") as f:
                f.write(serialized_state)
        except IOError as e:
            raise WriteError(e)

    def read(self, state_type):
        """Read a protocol buffer from the state file."""
        if not os.path.exists(self.__state_file):
            raise FileNotFoundError("No such file: %s" % self.__state_file )
        try:
            with open(self.__state_file, "rb") as f:
                serialized_state = f.read()
        except IOError as e:
            raise ReadError(e)
        state = state_type()
        try:
            state.ParseFromString(serialized_state)
            return state
        except message.DecodeError as e:
            raise CorruptStateError(e)



"""RFC 6962 client API."""
import base64
import json

from ct.crypto import verify
from ct.proto import client_pb2
import gflags
import logging
import requests
import urllib
import urlparse

FLAGS = gflags.FLAGS

gflags.DEFINE_integer("entry_fetch_batch_size", 1000, "Maximum number of "
                      "entries to attempt to fetch in one request.")

gflags.DEFINE_integer("get_entries_max_retries", 10, "Number of retries after "
                      "which get-entries simply fails.")


class Error(Exception):
    pass


class ClientError(Error):
    pass


class HTTPError(Error):
    """Connection failed, or returned an error."""
    pass


class HTTPClientError(HTTPError):
    """HTTP 4xx."""
    pass


class HTTPServerError(HTTPError):
    """HTTP 5xx."""
    pass


class InvalidRequestError(Error):
    """Request does not comply with the CT protocol."""
    pass


class InvalidResponseError(Error):
    """Response does not comply with the CT protocol."""
    pass


###############################################################################
#                    Common utility methods and constants.                    #
###############################################################################

_GET_STH_PATH = "ct/v1/get-sth"
_GET_ENTRIES_PATH = "ct/v1/get-entries"
_GET_STH_CONSISTENCY_PATH = "ct/v1/get-sth-consistency"
_GET_PROOF_BY_HASH_PATH = "ct/v1/get-proof-by-hash"
_GET_ROOTS_PATH = "ct/v1/get-roots"
_GET_ENTRY_AND_PROOF_PATH = "ct/v1/get-entry-and-proof"
_ADD_CHAIN = "ct/v1/add-chain"


def _parse_sth(sth_body):
    """Parse a serialized STH JSON response."""
    sth_response = client_pb2.SthResponse()
    try:
        sth = json.loads(sth_body)
        sth_response.timestamp = sth["timestamp"]
        sth_response.tree_size = sth["tree_size"]
        sth_response.sha256_root_hash = base64.b64decode(sth[
            "sha256_root_hash"])
        sth_response.tree_head_signature = base64.b64decode(sth[
            "tree_head_signature"])
        # TypeError for base64 decoding, TypeError/ValueError for invalid
        # JSON field types, KeyError for missing JSON fields.
    except (TypeError, ValueError, KeyError) as e:
        raise InvalidResponseError("Invalid STH %s\n%s" % (sth_body, e))
    return sth_response


def _parse_entry(json_entry):
    """Convert a json array element to an EntryResponse."""
    entry_response = client_pb2.EntryResponse()
    try:
        entry_response.leaf_input = base64.b64decode(
            json_entry["leaf_input"])
        entry_response.extra_data = base64.b64decode(
            json_entry["extra_data"])
    except (TypeError, ValueError, KeyError) as e:
        raise InvalidResponseError("Invalid entry: %s\n%s" % (json_entry, e))
    return entry_response


def _parse_entries(entries_body, expected_response_size):
    """Load serialized JSON response.

    Args:
        entries_body: received entries.
        expected_response_size: number of entries requested. Used to validate
            the response.

    Returns:
        a list of client_pb2.EntryResponse entries.

    Raises:
        InvalidResponseError: response not valid.
    """
    try:
        response = json.loads(entries_body)
    except ValueError as e:
        raise InvalidResponseError("Invalid response %s\n%s" %
                                   (entries_body, e))
    try:
        entries = iter(response["entries"])
    except (TypeError, KeyError) as e:
        raise InvalidResponseError("Invalid response: expected "
                                   "an array of entries, got %s\n%s)" %
                                   (response, e))
    # Logs MAY honor requests where 0 <= "start" < "tree_size" and
    # "end" >= "tree_size" by returning a partial response covering only
    # the valid entries in the specified range.
    # Logs MAY restrict the number of entries that can be retrieved per
    # "get-entries" request.  If a client requests more than the
    # permitted number of entries, the log SHALL return the maximum
    # number of entries permissible. (RFC 6962)
    #
    # Therefore, we cannot assume we get exactly the expected number of
    # entries. However if we get none, or get more than expected, then
    # we discard the response and raise.
    response_size = len(response["entries"])
    if not response_size or response_size > expected_response_size:
        raise InvalidResponseError("Invalid response: requested %d entries,"
                                   "got %d entries" %
                                   (expected_response_size, response_size))
    return [_parse_entry(e) for e in entries]

def _parse_consistency_proof(response, servername):
    try:
        response = json.loads(response)
        consistency = [base64.b64decode(u) for u in response["consistency"]]
    except (TypeError, ValueError, KeyError) as e:
        raise InvalidResponseError(
              "%s returned invalid data: expected a base64-encoded "
              "consistency proof, got %s"
              "\n%s" % (servername, response, e))
    return consistency

# A class that we can mock out to generate fake responses.
class RequestHandler(object):
    """HTTPS requests."""
    def __init__(self, connection_timeout=60, ca_bundle=True, num_retries=None):
        self._timeout = connection_timeout
        self._ca_bundle = ca_bundle
        # Explicitly check for None as num_retries being 0 is valid.
        if num_retries is None:
            num_retries = FLAGS.get_entries_max_retries
        self._num_retries = num_retries

    def __repr__(self):
        return "%r()" % self.__class__.__name__

    def __str__(self):
        return "%r()" % self.__class__.__name__

    def get_response(self, uri, params=None):
        """Get an HTTP response for a GET request."""
        uri_with_params = self._uri_with_params(uri, params)
        num_get_attempts = self._num_retries + 1
        while num_get_attempts > 0:
            try:
                return requests.get(uri, params=params, timeout=self._timeout,
                                    verify=self._ca_bundle)
            except requests.exceptions.ConnectionError as e:
                # Re-tries regardless of the error.
                # Cannot distinguish between an incomplete read and other
                # transient (or permanent) errors when using requests.
                num_get_attempts = num_get_attempts - 1
                logging.info("Retrying fetching %s, error %s" % (
                        uri_with_params, e))
        raise HTTPError(
              "Connection to %s failed too many times." % uri_with_params)

    def post_response(self, uri, post_data):
        try:
            return requests.post(uri, data=json.dumps(post_data),
                                 timeout=self._timeout, verify=self._ca_bundle)
        except requests.exceptions.RequestException as e:
            raise HTTPError("POST to %s failed: %s" % (uri, e))

    @staticmethod
    def check_response_status(code, reason, content='', headers=''):
        if code == 200:
            return
        elif 400 <= code < 500:
            raise HTTPClientError("%s (%s) %s" % (reason, content, headers))
        elif 500 <= code < 600:
            raise HTTPServerError("%s (%s) %s" % (reason, content, headers))
        else:
            raise HTTPError("%s (%s) %s" % (reason, content, headers))

    @staticmethod
    def _uri_with_params(uri, params=None):
        if not params:
            return uri
        components = list(urlparse.urlparse(uri))
        if params:
            # Update the URI query, which is at index 4 of the tuple.
            components[4] = urllib.urlencode(params)
        return urlparse.urlunparse(components)

    def get_response_body(self, uri, params=None):
        response = self.get_response(uri, params=params)
        self.check_response_status(response.status_code, response.reason,
                                   response.content, response.headers)
        return response.content

    def post_response_body(self, uri, post_data=None):
        response = self.post_response(uri, post_data=post_data)
        self.check_response_status(response.status_code, response.reason,
                                   response.content, response.headers)
        return response.content


###############################################################################
#                         The synchronous log client.                         #
###############################################################################


class LogClient(object):
    """HTTP client for talking to a CT log."""

    """Create a new log client.

    Args:
        uri: The CT Log URI to communicate with.
        handler: A custom RequestHandler to use. If not specified, a new one
        will be created.
        connection_timeout: Timeout (in seconds) for all GET and POST requests.
        ca_bundle: True or a file path containing a set of CA roots. See
        Requests documentation for more information:
        http://docs.python-requests.org/en/latest/user/advanced/#ssl-cert-verification
        Note that a false-y value is not allowed.
    """
    def __init__(self, uri, handler=None, connection_timeout=60,
                 ca_bundle=True):
        self._uri = uri
        if not ca_bundle:
          raise ClientError("Refusing to turn off SSL certificate checking.")
        if handler:
          self._request_handler = handler
        else:
          self._request_handler = RequestHandler(connection_timeout, ca_bundle)

    def __repr__(self):
        return "%r(%r)" % (self.__class__.__name__, self._request_handler)

    def __str__(self):
        return "%s(%s)" % (self.__class__.__name__, self._request_handler.uri)

    @property
    def servername(self):
        return self._uri

    def _req_body(self, path, params=None):
        return self._request_handler.get_response_body(self._uri + "/" + path,
                                                       params=params)

    def _post_req_body(self, path, post_data=None):
        return self._request_handler.post_response_body(
            self._uri + "/" + path, post_data=post_data)

    def _parse_sct(self, sct_response):
        sct_data = json.loads(sct_response)
        try:
            sct = client_pb2.SignedCertificateTimestamp()
            sct_version = sct_data["sct_version"]
            if sct_version != 0:
                raise InvalidResponseError(
                    "Unknown SCT version: %d" % sct_version)
            sct.version = client_pb2.V1
            sct.id.key_id = base64.b64decode(sct_data["id"])
            sct.timestamp = sct_data["timestamp"]
            hash_algorithm, sig_algorithm, sig_data = verify.decode_signature(
                base64.b64decode(sct_data["signature"]))
            sct.signature.hash_algorithm = hash_algorithm
            sct.signature.sig_algorithm = sig_algorithm
            sct.signature.signature = sig_data
            return sct
        except KeyError as e:
            raise InvalidResponseError("SCT Missing field: %s" % e)

    def get_sth(self):
        """Get the current Signed Tree Head.

        Returns:
            a ct.proto.client_pb2.SthResponse proto.

        Raises:
            HTTPError, HTTPClientError, HTTPServerError: connection failed.
                For logs that honour HTTP status codes, HTTPClientError (a 4xx)
                should never happen.
            InvalidResponseError: server response is invalid for the given
                                  request.
        """
        sth = self._req_body(_GET_STH_PATH)
        return _parse_sth(sth)

    def get_entries(self, start, end, batch_size=0):
        """Retrieve log entries.

        Args:
            start     : index of first entry to retrieve.
            end       : index of last entry to retrieve.
            batch_size: max number of entries to fetch in one go.

        Yields:
            ct.proto.client_pb2.EntryResponse protos.

        Raises:
            HTTPError, HTTPClientError, HTTPServerError: connection failed,
                or returned an error. HTTPClientError can happen when
                [start, end] is not a valid range for this log.
            InvalidRequestError: invalid request range (irrespective of log).
            InvalidResponseError: server response is invalid for the given
                                  request
        Caller is responsible for ensuring that (start, end) is a valid range
        (by retrieving an STH first), otherwise a HTTPClientError may occur.
        """
        # Catch obvious mistakes here.
        if start < 0 or end < 0 or start > end:
            raise InvalidRequestError("Invalid range [%d, %d]" % (start, end))

        batch_size = batch_size or FLAGS.entry_fetch_batch_size
        while start <= end:
            # Note that an HTTPError may occur here if the log does not have the
            # requested range of entries available. RFC 6962 says:
            # "Any errors will be returned as HTTP 4xx or 5xx responses, with
            # human-readable error messages."
            # There is thus no easy way to distinguish this case from other
            # errors.
            first = start
            last = min(start + batch_size - 1, end)
            response = self._req_body(_GET_ENTRIES_PATH,
                                      params={"start": first, "end": last})
            entries = _parse_entries(response, last - first + 1)
            for entry in entries:
                yield entry
            # If we got less entries than requested, then we don't know whether
            # the log imposed a batch limit or ran out of entries, so we keep
            # trying until we get all entries, or an error response.
            start += len(entries)

    def get_sth_consistency(self, old_size, new_size):
        """Retrieve a consistency proof.

        Args:
            old_size  : size of older tree.
            new_size  : size of newer tree.

        Returns:
            list of raw hashes (bytes) forming the consistency proof

        Raises:
            HTTPError, HTTPClientError, HTTPServerError: connection failed,
                or returned an error. HTTPClientError can happen when
                (old_size, new_size) are not valid for this log (e.g. greater
                than the size of the log).
            InvalidRequestError: invalid request size (irrespective of log).
            InvalidResponseError: server response is invalid for the given
                                  request
        Caller is responsible for ensuring that (old_size, new_size) are valid
        (by retrieving an STH first), otherwise a HTTPClientError may occur.
        """
        if old_size > new_size:
            raise InvalidRequestError(
                "old > new: %s >= %s" % (old_size, new_size))

        if old_size < 0 or new_size < 0:
            raise InvalidRequestError(
                "both sizes must be >= 0: %s, %s" % (old_size, new_size))

        # don't need to contact remote server for trivial proofs:
        # - empty tree is consistent with everything
        # - everything is consistent with itself
        if old_size == 0 or old_size == new_size:
            return []

        response = self._req_body(_GET_STH_CONSISTENCY_PATH,
                                  params={"first": old_size,
                                          "second": new_size})

        return _parse_consistency_proof(response, self.servername)

    def get_proof_by_hash(self, leaf_hash, tree_size):
        """Retrieve an audit proof by leaf hash.

        Args:
            leaf_hash: hash of the leaf input (as raw binary string).
            tree_size: size of the tree on which to base the proof.

        Returns:
            a client_pb2.ProofByHashResponse containing the leaf index
            and the Merkle tree audit path nodes (as binary strings).

        Raises:
            HTTPError, HTTPClientError, HTTPServerError: connection failed,
            HTTPClientError can happen when leaf_hash is not present in the
                log tree of the given size.
            InvalidRequestError: invalid request (irrespective of log).
            InvalidResponseError: server response is invalid for the given
                                  request.
        """
        if tree_size <= 0:
            raise InvalidRequestError("Tree size must be positive (got %d)" %
                                      tree_size)

        leaf_hash = base64.b64encode(leaf_hash)
        response = self._req_body(_GET_PROOF_BY_HASH_PATH,
                                  params={"hash": leaf_hash,
                                          "tree_size": tree_size})
        response = json.loads(response)

        proof_response = client_pb2.ProofByHashResponse()
        try:
            proof_response.leaf_index = response["leaf_index"]
            proof_response.audit_path.extend(
                [base64.b64decode(u) for u in response["audit_path"]])
        except (TypeError, ValueError, KeyError) as e:
            raise InvalidResponseError(
                "%s returned invalid data: expected a base64-encoded "
                "audit proof, got %s"
                "\n%s" % (self.servername, response, e))

        return proof_response

    def get_entry_and_proof(self, leaf_index, tree_size):
        """Retrieve an entry and its audit proof by index.

        Args:
            leaf_index: index of the entry.
            tree_size: size of the tree on which to base the proof.

        Returns:
            a client_pb2.EntryAndProofResponse containing the entry
            and the Merkle tree audit path nodes (as binary strings).

        Raises:
            HTTPError, HTTPClientError, HTTPServerError: connection failed,
            HTTPClientError can happen when tree_size is not a valid size
                for this log.
            InvalidRequestError: invalid request (irrespective of log).
            InvalidResponseError: server response is invalid for the given
                                  request.
        """
        if tree_size <= 0:
            raise InvalidRequestError("Tree size must be positive (got %d)" %
                                      tree_size)

        if leaf_index < 0 or leaf_index >= tree_size:
            raise InvalidRequestError("Leaf index must be smaller than tree "
                                      "size (got index %d vs size %d" %
                                      (leaf_index, tree_size))

        response = self._req_body(_GET_ENTRY_AND_PROOF_PATH,
                                  params={"leaf_index": leaf_index,
                                          "tree_size": tree_size})
        response = json.loads(response)

        entry_response = client_pb2.EntryAndProofResponse()
        try:
            entry_response.entry.CopyFrom(_parse_entry(response))
            entry_response.audit_path.extend(
                [base64.b64decode(u) for u in response["audit_path"]])
        except (TypeError, ValueError, KeyError) as e:
            raise InvalidResponseError(
                "%s returned invalid data: expected an entry and proof, got %s"
                "\n%s" % (self.servername, response, e))

        return entry_response

    def get_roots(self):
        """Retrieve currently accepted root certificates.

        Returns:
            a list of certificates (as raw binary strings).

        Raises:
            HTTPError, HTTPClientError, HTTPServerError: connection failed,
                or returned an error. For logs that honour HTTP status codes,
                HTTPClientError (a 4xx) should never happen.
            InvalidResponseError: server response is invalid for the given
                                  request.
        """
        response = self._req_body(_GET_ROOTS_PATH)
        response = json.loads(response)
        try:
            return [base64.b64decode(u)for u in response["certificates"]]
        except (TypeError, ValueError, KeyError) as e:
            raise InvalidResponseError(
                "%s returned invalid data: expected a list od base64-encoded "
                "certificates, got %s\n%s" % (self.servername, response, e))

    def add_chain(self, certs_list):
        """Adds the given chain of certificates.

        Args:
            certs_list: A list of DER-encoded certificates to add.

        Returns:
            The SCT for the certificate.

        Raises:
            HTTPError, HTTPClientError, HTTPServerError: connection failed.
                For logs that honour HTTP status codes, HTTPClientError (a 4xx)
                should never happen.
            InvalidResponseError: server response is invalid for the given
                                  request.
        """
        sct_data = self._post_req_body(
            _ADD_CHAIN,
            {'chain': [base64.b64encode(certificate) for certificate in certs_list]})
        return self._parse_sct(sct_data)



#!/usr/bin/env python

import collections
import functools
import multiprocessing
import traceback
import Queue

from ct.client import entry_decoder
from ct.client import log_client
from ct.crypto import cert
from ct.crypto import error
from ct.proto import client_pb2


# Messages types:
# Special queue messages to stop the subprocesses.
_WORKER_STOPPED = "WORKER_STOPPED"
_ERROR_PARSING_ENTRY = "ERROR_PARSING_ENTRY"
_ENTRY_MATCHING = "ENTRY_MATCHING"
_PROGRESS_REPORT = "PROGRESS_REPORT"


class QueueMessage(object):
    def __init__(self, msg_type, msg=None, certificates_scanned=1,
                 matcher_output=None):
        self.msg_type = msg_type
        self.msg = msg
        # Number of certificates scanned.
        self.certificates_scanned = certificates_scanned
        self.matcher_output = matcher_output


# This is only used on the entries input queue
_STOP_WORKER = "STOP_WORKER"

_BATCH_SIZE = 1000

def process_entries(entry_queue, output_queue, match_callback):
    stopped = False
    total_processed = 0
    while not stopped:
        count, entry = entry_queue.get()
        if entry == _STOP_WORKER:
            stopped = True
            # Each worker signals when they've picked up their
            # "STOP_WORKER" message.
            output_queue.put(QueueMessage(
                _WORKER_STOPPED,
                certificates_scanned=total_processed))
        else:
            entry_response = client_pb2.EntryResponse()
            entry_response.ParseFromString(entry)
            parsed_entry = entry_decoder.decode_entry(entry_response)
            ts_entry = parsed_entry.merkle_leaf.timestamped_entry
            total_processed += 1
            c = None
            if ts_entry.entry_type == client_pb2.X509_ENTRY:
                der_cert = ts_entry.asn1_cert
            else:
                # The original, signed precertificate.
                der_cert = (parsed_entry.extra_data.precert_chain_entry.pre_certificate)
            try:
                c = cert.Certificate(der_cert)
            except error.Error as e:
                try:
                    c = cert.Certificate(der_cert, strict_der=False)
                except error.Error as e:
                    output_queue.put(QueueMessage(
                        _ERROR_PARSING_ENTRY,
                        "Error parsing entry %d:\n%s" %
                        (count, e)))
                else:
                    output_queue.put(QueueMessage(
                        _ERROR_PARSING_ENTRY,
                        "Entry %d failed strict parsing:\n%s" %
                        (count, c)))
            except Exception as e:
                print "Unknown parsing failure for entry %d:\n%s" % (
                    count, e)
                traceback.print_exc()
                output_queue.put(QueueMessage(
                    _ERROR_PARSING_ENTRY,
                    "Entry %d failed parsing with an unknown error:\n%s" %
                    (count, e)))
            if c:
                match_result = match_callback(
                        c, ts_entry.entry_type, parsed_entry.extra_data, count)
                if match_result:
                    output_queue.put(QueueMessage(
                            _ENTRY_MATCHING,
                            "Entry %d:\n%s" % (count, c),
                            matcher_output=match_result))
            if not total_processed % _BATCH_SIZE:
                output_queue.put(QueueMessage(
                    _PROGRESS_REPORT,
                    "Scanned %d entries" % total_processed,
                    certificates_scanned=_BATCH_SIZE))

def _scan(entry_queue, log_url, range_description):
    range_start, range_end = range_description
    range_end -= 1
    client = log_client.LogClient(log_url)
    try:
        entries = client.get_entries(range_start, range_end)
        scanned = range_start
        for entry in entries:
            # Can't pickle protocol buffers with protobuf module version < 2.5.0
            # (https://code.google.com/p/protobuf/issues/detail?id=418)
            # so send serialized entry.
            entry_queue.put((scanned, entry.SerializeToString()))
            scanned += 1
    except Exception as e:
        print "Exception when fetching range %d to %d:\n%s" % (
            range_start, range_end, e)
        traceback.print_exc()

ScanResults = collections.namedtuple(
    'ScanResults', ['total', 'matches', 'errors'])


def _get_tree_size(log_url):
    client = log_client.LogClient(log_url)
    sth = client.get_sth()
    print "Got STH: %s" % sth
    return sth.tree_size


def _send_stop_to_workers(to_queue, num_instances):
    for _ in range(num_instances):
        to_queue.put((0, _STOP_WORKER))


def _process_worker_messages(
    workers_input_queue, workers_output_queue, scanners_done, num_workers,
    matcher_output_handler):
    total_scanned = 0
    total_matches = 0
    total_errors = 0
    scan_progress = 0

    workers_done = 0
    stop_sent = False
    while workers_done < num_workers:
        try:
            msg = workers_output_queue.get(block=True, timeout=3)
            if msg.msg_type == _WORKER_STOPPED:
                workers_done += 1
                total_scanned += msg.certificates_scanned
            elif msg.msg_type == _ERROR_PARSING_ENTRY:
                total_errors += 1
            elif msg.msg_type == _ENTRY_MATCHING:
                total_matches += 1
                if matcher_output_handler:
                    matcher_output_handler(msg.matcher_output)
            elif (msg.msg_type == _PROGRESS_REPORT and
                  msg.certificates_scanned > 0):
                scan_progress += msg.certificates_scanned
                print msg.msg, " Total: %d" % scan_progress
            else:
                print msg.msg
        except Queue.Empty:
            are_active = ""
            if scanners_done.value:
                are_active = "NOT"
            print "Scanners are %s active, Workers done: %d" % (
                are_active, workers_done)
        # Done handling the message, now let's check if the scanners
        # are done and if so stop the workers
        if scanners_done.value and not stop_sent:
            print "All scanners done, stopping."
            _send_stop_to_workers(workers_input_queue, num_workers)
            # To avoid re-sending stop
            stop_sent = True

    return ScanResults(total_scanned, total_matches, total_errors)

def scan_log(match_callback, log_url,total_processes=2,
             matcher_output_handler=None, start_entry=0):
    # (index, entry) tuples
    m = multiprocessing.Manager()
    R = 2
    assert total_processes >= R
    num_workers = total_processes // R
    num_scanners = total_processes - num_workers
    entry_queue = m.Queue(num_scanners * _BATCH_SIZE)
    output_queue = multiprocessing.Queue(10000)
    print "Allocating %d fetchers and %d processing workers" % (
        num_scanners, num_workers)

    tree_size = _get_tree_size(log_url)
    workers_done = multiprocessing.Value('b', 0)
    # Must use a flag rather than submitting STOP to the queue directly
    # since if the queue will be full there'll be a deadlock.
    def stop_workers_callback(_):
        workers_done.value = 1

    bound_scan = functools.partial(_scan, entry_queue, log_url)

    scan_start_range = range(start_entry, tree_size, _BATCH_SIZE)
    scan_range = zip(scan_start_range, scan_start_range[1:] + [tree_size])
    scanners_pool = multiprocessing.Pool(num_scanners)
    res = scanners_pool.map_async(bound_scan, scan_range,
                                  callback=stop_workers_callback)
    scanners_pool.close()

    workers = [
        multiprocessing.Process(
            target=process_entries,
            args=(entry_queue, output_queue, match_callback))
               for _ in range(num_workers)]
    for w in workers:
        w.start()

    try:
      res = _process_worker_messages(
          entry_queue, output_queue, workers_done, num_workers,
          matcher_output_handler)
    # Do not hang the interpreter upon ^C.
    except (KeyboardInterrupt, SystemExit):
        for w in workers:
            w.terminate()
        scanners_pool.terminate()
        m.shutdown()
        raise

    scanners_pool.join()
    for w in workers:
        w.join()
    m.shutdown()
    return res



"""RFC 6962 client API."""

from ct.client import log_client
from ct.client.db import database
import gflags
import logging
import random

from twisted.internet import defer
from twisted.internet import error
from twisted.internet import protocol
from twisted.internet import reactor as ireactor
from twisted.internet import task
from twisted.internet import threads
from twisted.python import failure
from twisted.web import client
from twisted.web import http
from twisted.web import iweb
from Queue import Queue
from zope.interface import implements


FLAGS = gflags.FLAGS

gflags.DEFINE_integer("max_fetchers_in_parallel", 100, "Maximum number of "
                      "concurrent fetches.")

gflags.DEFINE_integer("get_entries_retry_delay", 1, "Number of seconds after "
                      "which get-entries will be retried if it encountered "
                      "an error.")

gflags.DEFINE_integer("entries_buffer", 100000, "Size of buffer which stores "
                      "fetched entries before async log client is able to "
                      "return them. 100000 entries shouldn't take more "
                      "than 600 Mb of memory.")

gflags.DEFINE_integer("response_buffer_size_bytes", 50 * 1000 * 1000, "Maximum "
                      "size of a single response buffer. Should be set such "
                      "that a get_entries response comfortably fits in the "
                      "the buffer. A typical log entry is expected to be < "
                      "10kB.")

gflags.DEFINE_bool("persist_entries", True, "Cache entries on disk.")

class HTTPConnectionError(log_client.HTTPError):
    """Connection failed."""
    pass


class HTTPResponseSizeExceededError(log_client.HTTPError):
    """HTTP response exceeded maximum permitted size."""
    pass


###############################################################################
#                       The asynchronous twisted log client.                  #
###############################################################################


class ResponseBodyHandler(protocol.Protocol):
    """Response handler for HTTP requests."""

    def __init__(self, finished):
        """Initialize the one-off response handler.

        Args:
            finished: a deferred that will be fired with the body when the
                complete response has been received; or with an error when the
                connection is lost.
        """
        self._finished = finished

    def connectionMade(self):
        self._buffer = []
        self._len = 0
        self._overflow = False

    def dataReceived(self, data):
        self._len += len(data)
        if self._len > FLAGS.response_buffer_size_bytes:
            # Note this flag has to be set *before* calling loseConnection()
            # to ensure connectionLost gets called with the flag set.
            self._overflow = True
            self.transport.loseConnection()
        else:
            self._buffer.append(data)

    def connectionLost(self, reason):
        if self._overflow:
            self._finished.errback(HTTPResponseSizeExceededError(
                "Connection aborted: response size exceeded %d bytes" %
                FLAGS.response_buffer_size_bytes))
        elif not reason.check(*(error.ConnectionDone, client.ResponseDone,
                                http.PotentialDataLoss)):
            self._finished.errback(HTTPConnectionError(
                "Connection lost (received %d bytes)" % self._len))
        else:
            body = "".join(self._buffer)
            self._finished.callback(body)


class AsyncRequestHandler(object):
    """A helper for asynchronous response body delivery."""

    def __init__(self, agent):
        self._agent = agent

    @staticmethod
    def _response_cb(response):
        try:
            log_client.RequestHandler.check_response_status(
                    response.code, response.phrase,
                    list(response.headers.getAllRawHeaders()))
        except log_client.HTTPError as e:
            return failure.Failure(e)
        finished = defer.Deferred()
        response.deliverBody(ResponseBodyHandler(finished))
        return finished

    @staticmethod
    def _make_request(path, params):
        if not params:
            return path
        return path + "?" + "&".join(["%s=%s" % (key, value)
                                      for key, value in params.iteritems()])

    def get(self, path, params=None):
        d = self._agent.request("GET", self._make_request(path, params))
        d.addCallback(self._response_cb)
        return d


class EntryProducer(object):
    """A push producer for log entries."""
    implements(iweb.IBodyProducer)

    def __init__(self, handler, reactor, uri, start, end,
                 batch_size, entries_db=None):
        self._handler = handler
        self._reactor = reactor
        self._uri = uri
        self._entries_db = entries_db
        self._consumer = None

        assert 0 <= start <= end
        self._start = start
        self._end = end
        self._current = self._start
        self._batch_size = batch_size
        self._batches = Queue()
        self._currently_fetching = 0
        self._currently_stored = 0
        self._last_fetching = self._current
        self._max_currently_fetching = (FLAGS.max_fetchers_in_parallel *
                                        self._batch_size)
        # Required attribute of the interface.
        self.length = iweb.UNKNOWN_LENGTH
        self.min_delay = FLAGS.get_entries_retry_delay

    @property
    def finished(self):
        return self._current > self._end

    def __fail(self, failure):
        if not self._stopped:
            self.stopProducing()
            self._done.errback(failure)

    @staticmethod
    def _calculate_retry_delay(retries):
        """Calculates delay based on number of retries which already happened.

        Random is there, so we won't attack server lots of requests exactly
        at the same time, and 1.3 is nice constant for exponential back-off."""
        return ((0.4 + random.uniform(0.3, 0.6)) * FLAGS.get_entries_retry_delay
                * 1.4**retries)

    def _response_eb(self, failure, first, last, retries):
        """Error back for HTTP errors"""
        if not self._paused:
            # if it's not last retry and failure wasn't our fault we retry
            if (retries < FLAGS.get_entries_max_retries and
                not failure.check(log_client.HTTPClientError)):
                logging.info("Retrying get-entries for range <%d, %d> retry: %d"
                             % (first, last, retries))
                d = task.deferLater(self._reactor,
                                  self._calculate_retry_delay(retries),
                                  self._fetch_parsed_entries,
                                  first, last)
                d.addErrback(self._response_eb, first, last, retries + 1)
                return d
            else:
                self.__fail(failure)

    def _fetch_eb(self, failure):
        """Error back for errors after getting result of a request
        (InvalidResponse)"""
        self.__fail(failure)

    def _write_pending(self):
        d = defer.Deferred()
        d.callback(None)
        if self._pending:
            self._current += len(self._pending)
            self._currently_stored -= len(self._pending)
            d = self._consumer.consume(self._pending)
            self._pending = None
        return d

    def _batch_completed(self, result):
        self._currently_fetching -= len(result)
        self._currently_stored += len(result)
        return result

    def _store_batch(self, entry_batch, start_index):
        assert self._entries_db
        d = threads.deferToThread(self._entries_db.store_entries,
                                  enumerate(entry_batch, start_index))
        d.addCallback(lambda _: entry_batch)
        return d

    def _get_entries_from_db(self, first, last):
        if FLAGS.persist_entries and self._entries_db:
            d = threads.deferToThread(self._entries_db.scan_entries, first, last)
            d.addCallbacks(lambda entries: list(entries))
            d.addErrback(lambda fail: fail.trap(database.KeyError) and None)
            return d
        else:
            d = defer.Deferred()
            d.callback(None)
            return d

    def _fetch_parsed_entries(self, first, last):
        # first check in database
        d = self._get_entries_from_db(first, last)
        d.addCallback(self._sub_fetch_parsed_entries, first, last)
        return d

    def _sub_fetch_parsed_entries(self, entries, first, last):
        # it's not the best idea to attack server with many requests exactly at
        # the same time, so requests are sent after slight delay.
        if not entries:
            request = task.deferLater(self._reactor,
                                      self._calculate_retry_delay(0),
                                      self._handler.get,
                                      self._uri + "/" +
                                      log_client._GET_ENTRIES_PATH,
                                      params={"start": str(first),
                                              "end": str(last)})
            request.addCallback(log_client._parse_entries, last - first + 1)
            if self._entries_db and FLAGS.persist_entries:
                request.addCallback(self._store_batch, first)
            entries = request
        else:
            deferred_entries = defer.Deferred()
            deferred_entries.callback(entries)
            entries = deferred_entries
        return entries

    def _create_next_request(self, first, last, entries, retries):
        d = self._fetch_parsed_entries(first, last)
        d.addErrback(self._response_eb, first, last, retries)
        d.addCallback(lambda result: (entries + result, len(result)))
        d.addCallback(self._fetch, first, last, retries)
        return d

    def _fetch(self, result, first, last, retries):
        entries, last_fetched_entries_count = result
        next_range_start = first + last_fetched_entries_count
        if next_range_start > last:
            return entries
        return self._create_next_request(next_range_start, last,
                                         entries, retries)

    def _create_fetch_deferred(self, first, last, retries=0):
        d = defer.Deferred()
        d.addCallback(self._fetch, first, last, retries)
        d.addCallback(self._batch_completed)
        d.addErrback(self._fetch_eb)
        d.callback(([], 0))
        return d

    @defer.deferredGenerator
    def produce(self):
        """Produce entries."""
        while not self._paused:
            wfd = defer.waitForDeferred(self._write_pending())
            yield wfd
            wfd.getResult()

            if self.finished:
                self.finishProducing()
                return
            first = self._last_fetching
            while (self._currently_fetching <= self._max_currently_fetching and
                   self._last_fetching <= self._end and
                   self._currently_stored <= FLAGS.entries_buffer):
                last = min(self._last_fetching + self._batch_size - 1, self._end,
                   self._last_fetching + self._max_currently_fetching
                           - self._currently_fetching + 1)
                self._batches.put(self._create_fetch_deferred(first, last))
                self._currently_fetching += last - first + 1
                first = last + 1
                self._last_fetching = first

            wfd = defer.waitForDeferred(self._batches.get())
            # Pause here until the body of the response is available.
            yield wfd
            # The producer may have been paused while waiting for the response,
            # or errored out upon receiving it: do not write the entries out
            # until after the next self._paused check.
            self._pending = wfd.getResult()

    def startProducing(self, consumer):
        """Start producing entries.

        The producer writes EntryResponse protos to the consumer in batches,
        until all entries have been received, or an error occurs.

        Args:
            consumer: the consumer to write to.

        Returns:
           a deferred that fires when no more entries will be written.
           Upon success, this deferred fires number of produced entries or
           None if production wasn't successful. Upon failure, this deferred
           fires with the appropriate HTTPError.

        Raises:
            RuntimeError: consumer already registered.
        """
        if self._consumer:
            raise RuntimeError("Producer already has a consumer registered")
        self._consumer = consumer
        self._stopped = False
        self._paused = True
        self._pending = None
        self._done = defer.Deferred()
        # An IBodyProducer should start producing immediately, without waiting
        # for an explicit resumeProducing() call.
        task.deferLater(self._reactor, 0, self.resumeProducing)
        return self._done

    def pauseProducing(self):
        self._paused = True

    def resumeProducing(self):
        if self._paused and not self._stopped:
            self._paused = False
            d = self.produce()
            d.addErrback(self.finishProducing)

    def stopProducing(self):
        self._paused = True
        self._stopped = True

    def finishProducing(self, failure=None):
        self.stopProducing()
        if not failure:
            self._done.callback(self._end - self._start + 1)
        else:
            self._done.errback(failure)


class AsyncLogClient(object):
    """A twisted log client."""

    def __init__(self, agent, uri, entries_db=None, reactor=ireactor):

        """Initialize the client.

        If entries_db is specified and flag persist_entries is true, get_entries
        will return stored entries.

        Args:
            agent: the agent to use.
            uri: the uri of the log.
            entries_db: object that conforms TempDB API
            reactor: the reactor to use. Default is twisted.internet.reactor.
        """
        self._handler = AsyncRequestHandler(agent)
        #twisted expects bytes, so if uri is unicode we have to change encoding
        self._uri = uri.encode('ascii')
        self._reactor = reactor
        self._entries_db = entries_db

    @property
    def servername(self):
        return self._uri

    def get_sth(self):
        """Get the current Signed Tree Head.

        Returns:
            a Deferred that fires with a ct.proto.client_pb2.SthResponse proto.

        Raises:
            HTTPError, HTTPConnectionError, HTTPClientError,
            HTTPResponseSizeExceededError, HTTPServerError: connection failed.
                For logs that honour HTTP status codes, HTTPClientError (a 4xx)
                should never happen.
            InvalidResponseError: server response is invalid for the given
                                  request.
        """
        deferred_result = self._handler.get(self._uri + "/" +
                                            log_client._GET_STH_PATH)
        deferred_result.addCallback(log_client._parse_sth)
        return deferred_result

    def get_entries(self, start, end, batch_size=0):
        """Retrieve log entries.

        Args:
            start: index of first entry to retrieve.
            end: index of last entry to retrieve.
            batch_size: max number of entries to fetch in one go.

        Returns:
            an EntryProducer for the given range.

        Raises:
            InvalidRequestError: invalid request range (irrespective of log).

        Caller is responsible for ensuring that (start, end) is a valid range
        (by retrieving an STH first), otherwise a HTTPClientError may occur
        during production.
        """
        # Catch obvious mistakes here.
        if start < 0 or end < 0 or start > end:
            raise log_client.InvalidRequestError(
                    "Invalid range [%d, %d]" % (start, end))
        batch_size = batch_size or FLAGS.entry_fetch_batch_size
        return EntryProducer(self._handler, self._reactor, self._uri,
                             start, end, batch_size, self._entries_db)

    def get_sth_consistency(self, old_size, new_size):
        """Retrieve a consistency proof.

        Args:
            old_size  : size of older tree.
            new_size  : size of newer tree.

        Returns:
            a Deferred that fires with list of raw hashes (bytes) forming the
            consistency proof

        Raises:
            HTTPError, HTTPClientError, HTTPServerError: connection failed,
                or returned an error. HTTPClientError can happen when
                (old_size, new_size) are not valid for this log (e.g. greater
                than the size of the log).
            InvalidRequestError: invalid request size (irrespective of log).
            InvalidResponseError: server response is invalid for the given
                                  request
        Caller is responsible for ensuring that (old_size, new_size) are valid
        (by retrieving an STH first), otherwise a HTTPClientError may occur.
        """
        if old_size > new_size:
            raise log_client.InvalidRequestError(
                "old > new: %s >= %s" % (old_size, new_size))

        if old_size < 0 or new_size < 0:
            raise log_client.InvalidRequestError(
                "both sizes must be >= 0: %s, %s" % (old_size, new_size))

        # don't need to contact remote server for trivial proofs:
        # - empty tree is consistent with everything
        # - everything is consistent with itself
        if old_size == 0 or old_size == new_size:
            d = defer.Deferred()
            d.callback([])
            return d

        deferred_response = self._handler.get(
                self._uri + "/" +
                log_client._GET_STH_CONSISTENCY_PATH,
                params={"first": old_size, "second": new_size})
        deferred_response.addCallback(log_client._parse_consistency_proof,
                                      self.servername)
        return deferred_response



"""TLS serialization."""

import math

from ct.proto import tls_options_pb2 as options
from google.protobuf import descriptor


class Error(Exception):
    pass


class TLSDecodingError(Error):
    """Decoding failed."""
    pass


class TLSEncodingError(Error):
    """Encoding failed."""
    pass


class TLSReader(object):
    """Read serialized TLS messages into a protocol buffer."""

    def __init__(self, serialized_buffer):
        # It would be nice to use BytesIO but it has no efficient way of
        # testing whether it's empty without advancing the position, so
        # we have to keep track of the position manually.
        self._buf = serialized_buffer
        self._pos = 0

    def _read_fixed_bytes(self, num_bytes):
        if self._pos + num_bytes > len(self._buf):
            raise TLSDecodingError("Buffer underrun: need %d bytes, have "
                                   "%d bytes" % (num_bytes,
                                                 len(self._buf) - self._pos))
        ret = self._buf[self._pos:self._pos + num_bytes]
        self._pos += num_bytes
        return ret

    def finished(self):
        return self._pos >= len(self._buf)

    def verify_finished(self):
        if not self.finished():
            raise TLSDecodingError("Bytes remaining in the buffer")

    def _read_uint(self, num_bytes):
        int_bytes = bytearray(self._read_fixed_bytes(num_bytes))
        ret = 0
        for b in int_bytes:
            ret <<= 8
            ret += b
        return ret

    def _read_bounded_uint(self, min_value, max_value):
        length_of_value = int(math.ceil(math.log(max_value + 1, 256)))
        value = self._read_uint(length_of_value)
        if value < min_value or value > max_value:
            raise TLSDecodingError("Value %d is out of range ([%d, %d])" %
                                   (value, min_value, max_value))
        return value

    def _read_uint32(self, opts):
        return self._read_uint(opts.bytes_in_use or 4)

    def _read_uint64(self, opts):
        return self._read_uint(opts.bytes_in_use or 8)

    def _read_enum(self, opts):
        if not opts.max_value:
            raise TypeError("Enum field has no maximum value")
        return self._read_bounded_uint(0, opts.max_value)

    def _read_var_bytes(self, min_length, max_length):
        length = self._read_bounded_uint(min_length, max_length)
        return self._read_fixed_bytes(length)

    def _read_bytes(self, opts):
        if opts.fixed_length:
            return self._read_fixed_bytes(opts.fixed_length)
        elif opts.max_length:
            return self._read_var_bytes(opts.min_length, opts.max_length)
        else:
            raise TypeError("Byte field has no length limit")

    def _get_read_method(self, field):
        if field.type == descriptor.FieldDescriptor.TYPE_UINT32:
            return self._read_uint32
        elif field.type == descriptor.FieldDescriptor.TYPE_UINT64:
            return self._read_uint64
        elif field.type == descriptor.FieldDescriptor.TYPE_ENUM:
            return self._read_enum
        elif field.type == descriptor.FieldDescriptor.TYPE_BYTES:
            return self._read_bytes
        else:
            raise TypeError("Field %s of type %d not supported" %
                            (field.name, field.type))

    def _read_repeated(self, message, field, opts):
        """Read a repeated field."""
        if not opts.max_total_length:
            raise TypeError("Repeated field %s has no length limit" %
                            field.name)
        # Recursive, naive.
        reader = TLSReader(self._read_var_bytes(opts.min_total_length,
                                                opts.max_total_length))

        target = getattr(message, field.name)

        if field.type == field.TYPE_MESSAGE:
            while not reader.finished():
                new_message = target.add()
                reader.read(new_message)
        else:
            if field.type == field.TYPE_ENUM:
                opts = field.enum_type.GetOptions().Extensions[
                    options.tls_enum_opts]
            # |reader| is another member of this class.
            # pylint: disable=protected-access
            read_method = reader._get_read_method(field)
            while not reader.finished():
                target.append(read_method(opts))

    def read(self, message):
        """Read from the buffer into the protocol buffer message."""
        # TODO(ekasper): probably better not to modify the
        # original message until we're guaranteed to succeed?
        fields = message.DESCRIPTOR.fields_by_number
        for i in sorted(fields):
            field = fields[i]
            opts = field.GetOptions().Extensions[options.tls_opts]
            if opts.skip:
                continue

            if opts.select_field:
                value = getattr(message, opts.select_field)
                if value != opts.select_value:
                    continue

            if field.label == field.LABEL_REPEATED:
                self._read_repeated(message, field, opts)

            elif field.type == field.TYPE_MESSAGE:
                self.read(getattr(message, field.name))

            else:
                if field.type == field.TYPE_ENUM:
                    opts = field.enum_type.GetOptions().Extensions[
                        options.tls_enum_opts]
                setattr(message, field.name,
                        self._get_read_method(field)(opts))

    @classmethod
    def decode(cls, buf, message):
        reader = cls(buf)
        reader.read(message)
        reader.verify_finished()


class TLSWriter(object):
    """Serialize protocol buffers into TLS wire format."""

    def __init__(self):
        self._buf = bytearray()

    def __len__(self):
        """Current length of the result."""
        return len(self._buf)

    def get_serialized_result(self):
        """Get the serialized contents."""
        return str(self._buf)

    def _write_uint(self, value, num_bytes):
        int_bytes = bytearray()
        for _ in range(num_bytes):
            int_bytes.append(value & 0xff)
            value >>= 8
        if value:
            raise TLSEncodingError("Value %d is too large to fit in %d bytes" %
                                   (value, num_bytes))
        int_bytes.reverse()
        self._buf.extend(int_bytes)

    def _write_bounded_uint(self, value, min_value, max_value):
        if value < min_value or value > max_value:
            raise TLSEncodingError("Value %d out of range ([%d, %d])" %
                                   (value, min_value, max_value))

        length_of_value = int(math.ceil(math.log(max_value + 1, 256)))
        self._write_uint(value, length_of_value)

    def _write_uint32(self, value, opts):
        self._write_uint(value, opts.bytes_in_use or 4)

    def _write_uint64(self, value, opts):
        self._write_uint(value, opts.bytes_in_use or 8)

    def _write_enum(self, value, opts):
        if not opts.max_value:
            raise TypeError("Enum field has no maximum value")
        self._write_bounded_uint(value, 0, opts.max_value)

    def _write_fixed_bytes(self, value, length):
        if len(value) != length:
            raise TLSEncodingError("Invalid value %s of length %d: required "
                                   "length is %d" % (value, len(value), length))
        self._buf.extend(value)

    def _write_var_bytes(self, value, min_length, max_length):
        self._write_bounded_uint(len(value), min_length, max_length)
        self._buf.extend(value)

    def _write_bytes(self, value, opts):
        if opts.fixed_length:
            return self._write_fixed_bytes(value, opts.fixed_length)
        elif opts.max_length:
            return self._write_var_bytes(value, opts.min_length,
                                         opts.max_length)
        else:
            raise TypeError("Byte field has no length limit")

    def _get_write_method(self, field):
        if field.type == descriptor.FieldDescriptor.TYPE_UINT32:
            return self._write_uint32
        elif field.type == descriptor.FieldDescriptor.TYPE_UINT64:
            return self._write_uint64
        elif field.type == descriptor.FieldDescriptor.TYPE_ENUM:
            return self._write_enum
        elif field.type == descriptor.FieldDescriptor.TYPE_BYTES:
            return self._write_bytes
        else:
            raise TypeError("Field %s of type %d not supported" %
                            (field.name, field.type))

    def _write_repeated(self, message, field, opts):
        """Write a repeated field."""
        writer = TLSWriter()
        if field.type == field.TYPE_MESSAGE:
            # Recursive, naive: could instead read ahead to determine
            # the total length first.
            for elem in getattr(message, field.name):
                writer.write(elem)
        else:
            if field.type == field.TYPE_ENUM:
                opts = field.enum_type.GetOptions().Extensions[
                    options.tls_enum_opts]
            # pylint: disable=protected-access
            write_method = writer._get_write_method(field)
            for elem in getattr(message, field.name):
                write_method(elem, opts)

        length = len(writer)
        self._write_bounded_uint(length, opts.min_total_length,
                                 opts.max_total_length)
        self._buf.extend(writer._buf)  # pylint: disable=protected-access

    def write(self, message):
        """Append a serialized message to the writer's buffer."""
        fields = message.DESCRIPTOR.fields_by_number
        for i in sorted(fields):
            field = fields[i]
            opts = field.GetOptions().Extensions[options.tls_opts]
            if opts.skip:
                continue

            if opts.select_field:
                value = getattr(message, opts.select_field)
                if value != opts.select_value:
                    continue

            if field.label == field.LABEL_REPEATED:
                self._write_repeated(message, field, opts)

            elif field.type == field.TYPE_MESSAGE:
                self.write(getattr(message, field.name))

            else:
                if field.type == field.TYPE_ENUM:
                    opts = field.enum_type.GetOptions().Extensions[
                        options.tls_enum_opts]
                self._get_write_method(field)(getattr(message, field.name),
                                              opts)

    @classmethod
    def encode(cls, message):
        writer = cls()
        writer.write(message)
        return writer.get_serialized_result()


def encode(message):
    return TLSWriter.encode(message)


def decode(buf, message):
    TLSReader.decode(buf, message)



from ct.serialization import tls_message
from ct.proto import client_pb2

def decode_entry(entry):
    parsed_entry = client_pb2.ParsedEntry()
    tls_message.decode(entry.leaf_input, parsed_entry.merkle_leaf)

    parsed_entry.extra_data.entry_type = (parsed_entry.merkle_leaf.
                                          timestamped_entry.entry_type)

    tls_message.decode(entry.extra_data, parsed_entry.extra_data)
    return parsed_entry



#!/usr/bin/env python
""" This is a util to dump the SCTs contained within a
    SignedCertificateTimestampList structure.
    This structure is used to represent a collection of SCTs being passed over
    a TLS handshake.  See RFC6962 section 3.3 for more details. """
import sys
from ct.proto import client_pb2
from ct.serialization import tls_message

def dump_sctlist(sct_list):
    """Prints the proto representation of the SCTs contained in sct_list.
       Arguments:
       sct_list the packed SignedCertificateTransparencyList structure.
    """
    tr = tls_message.TLSReader(sct_list)
    sctlist = client_pb2.SignedCertificateTimestampList()
    tr.read(sctlist)
    for s in sctlist.sct_list:
        sct = client_pb2.SignedCertificateTimestamp()
        tls_message.decode(s, sct)
        print sct

if __name__ == "__main__":
    if len(sys.argv) == 1:
        print "Usage: dump_sctlist.py <file_containing_sct_list>"
        sys.exit(1)

    with open(sys.argv[1]) as f:
        data = f.read()
    dump_sctlist(data)



#!/usr/bin/env python

import unittest

from collections import namedtuple
import hashlib
import math

from ct.crypto import error
from ct.crypto import merkle


class TreeHasherTest(unittest.TestCase):
    sha256_empty_hash = ("e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495"
                         "991b7852b855")
    sha256_leaves = [
        ("",
         "6e340b9cffb37a989ca544e6bb780a2c78901d3fb33738768511a30617afa01d"),
        ("101112131415161718191a1b1c1d1e1f",
         "3bfb960453ebaebf33727da7a1f4db38acc051d381b6da20d6d4e88f0eabfd7a")
        ]
    sha256_nodes = [
        ("000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f",
         "202122232425262728292a2b2c2d2e2f303132333435363738393a3b3c3d3e3f",
         "1a378704c17da31e2d05b6d121c2bb2c7d76f6ee6fa8f983e596c2d034963c57")]

    # array of bytestrings of the following literals in hex
    test_vector_leaves = ["".join(chr(int(n, 16)) for n in s.split()) for s in [
        "",
        "00",
        "10",
        "20 21",
        "30 31",
        "40 41 42 43",
        "50 51 52 53 54 55 56 57",
        "60 61 62 63 64 65 66 67 68 69 6a 6b 6c 6d 6e 6f",
    ]]

    test_vector_hashes = [
        "6e340b9cffb37a989ca544e6bb780a2c78901d3fb33738768511a30617afa01d",
        "fac54203e7cc696cf0dfcb42c92a1d9dbaf70ad9e621f4bd8d98662f00e3c125",
        "aeb6bcfe274b70a14fb067a5e5578264db0fa9b51af5e0ba159158f329e06e77",
        "d37ee418976dd95753c1c73862b9398fa2a2cf9b4ff0fdfe8b30cd95209614b7",
        "4e3bbb1f7b478dcfe71fb631631519a3bca12c9aefca1612bfce4c13a86264d4",
        "76e67dadbcdf1e10e1b74ddc608abd2f98dfb16fbce75277b5232a127f2087ef",
        "ddb89be403809e325750d3d263cd78929c2942b7942a34b77e122c9594a74c8c",
        "5dc9da79a70659a9ad559cb701ded9a2ab9d823aad2f4960cfe370eff4604328",
    ]

    def test_empty_hash(self):
        hasher = merkle.TreeHasher()
        self.assertEqual(hasher.hash_empty().encode("hex"),
                         TreeHasherTest.sha256_empty_hash)

    def test_hash_leaves(self):
        hasher = merkle.TreeHasher()
        for leaf, val in TreeHasherTest.sha256_leaves:
            self.assertEqual(hasher.hash_leaf(leaf.decode("hex")).encode("hex"),
                             val)

    def test_hash_children(self):
        hasher = merkle.TreeHasher()
        for left, right, val in  TreeHasherTest.sha256_nodes:
            self.assertEqual(hasher.hash_children(
                left.decode("hex"), right.decode("hex")).encode("hex"), val)

    def test_hash_full_invalid_index(self):
        hasher = merkle.TreeHasher()
        self.assertRaises(IndexError, hasher._hash_full, "abcd", -5, -1)
        self.assertRaises(IndexError, hasher._hash_full, "abcd", -1, 1)
        self.assertRaises(IndexError, hasher._hash_full, "abcd", 1, 5)
        self.assertRaises(IndexError, hasher._hash_full, "abcd", 2, 1)

    def test_hash_full_empty(self):
        hasher = merkle.TreeHasher()
        for i in xrange(0, 5):
            self.assertEqual(hasher._hash_full("abcd", i, i)[0].encode("hex"),
                              TreeHasherTest.sha256_empty_hash)

    def test_hash_full_tree(self):
        hasher = merkle.TreeHasher()
        self.assertEqual(hasher.hash_full_tree([]), hasher.hash_empty())
        l = iter(hasher.hash_leaf(c) for c in "abcde").next
        h = hasher.hash_children
        root_hash = h(h(h(l(), l()), h(l(), l())), l())
        self.assertEqual(hasher.hash_full_tree("abcde"), root_hash)

    def test_hash_full_tree_test_vector(self):
        hasher = merkle.TreeHasher()
        for i in xrange(len(TreeHasherTest.test_vector_leaves)):
            test_vector = TreeHasherTest.test_vector_leaves[:i+1]
            expected_hash = TreeHasherTest.test_vector_hashes[i].decode("hex")
            self.assertEqual(hasher.hash_full_tree(test_vector), expected_hash)


class HexTreeHasher(merkle.TreeHasher):
    def __init__(self, hashfunc=hashlib.sha256):
        self.hasher = merkle.TreeHasher(hashfunc)

    def hash_empty(self):
        return self.hasher.hash_empty().encode("hex")

    def hash_leaf(self, data):
        return self.hasher.hash_leaf(data.decode("hex")).encode("hex")

    def hash_children(self, left, right):
        return self.hasher.hash_children(left.decode("hex"),
                                         right.decode("hex")).encode("hex")


class CompactMerkleTreeTest(unittest.TestCase):

    def setUp(self):
        self.tree = merkle.CompactMerkleTree(HexTreeHasher())

    def test_extend_from_empty(self):
        for i in xrange(len(TreeHasherTest.test_vector_leaves)):
            test_vector = TreeHasherTest.test_vector_leaves[:i+1]
            expected_hash = TreeHasherTest.test_vector_hashes[i]
            self.tree = merkle.CompactMerkleTree()
            self.tree.extend(test_vector)
            self.assertEqual(self.tree.root_hash().encode("hex"), expected_hash)

    def test_push_subtree_1(self):
        for i in xrange(len(TreeHasherTest.test_vector_leaves)):
            test_vector = TreeHasherTest.test_vector_leaves[:i+1]
            self.tree = merkle.CompactMerkleTree()
            self.tree.extend(test_vector)
            self.tree._push_subtree(["test leaf"])
            self.assertEqual(len(self.tree), len(test_vector) + 1)

    def test_extend_from_partial(self):
        z = len(TreeHasherTest.test_vector_leaves)
        for i in xrange(z):
            self.tree = merkle.CompactMerkleTree()
            # add up to i
            test_vector = TreeHasherTest.test_vector_leaves[:i+1]
            expected_hash = TreeHasherTest.test_vector_hashes[i]
            self.tree.extend(test_vector)
            self.assertEqual(self.tree.root_hash().encode("hex"), expected_hash)
            # add up to z
            test_vector = TreeHasherTest.test_vector_leaves[i+1:]
            expected_hash = TreeHasherTest.test_vector_hashes[z-1]
            self.tree.extend(test_vector)
            self.assertEqual(self.tree.root_hash().encode("hex"), expected_hash)


class MerkleVerifierTest(unittest.TestCase):
    # (old_tree_size, new_tree_size, old_root, new_root, proof)
    # Test vectors lifted from the C++ branch.
    sha256_proofs = [
        (1, 1,
         "6e340b9cffb37a989ca544e6bb780a2c78901d3fb33738768511a30617afa01d",
         "6e340b9cffb37a989ca544e6bb780a2c78901d3fb33738768511a30617afa01d",
         []),
        (1, 8,
         "6e340b9cffb37a989ca544e6bb780a2c78901d3fb33738768511a30617afa01d",
         "5dc9da79a70659a9ad559cb701ded9a2ab9d823aad2f4960cfe370eff4604328",
         ["96a296d224f285c67bee93c30f8a309157f0daa35dc5b87e410b78630a09cfc7",
          "5f083f0a1a33ca076a95279832580db3e0ef4584bdff1f54c8a360f50de3031e",
          "6b47aaf29ee3c2af9af889bc1fb9254dabd31177f16232dd6aab035ca39bf6e4"]),
        (6, 8,
         "76e67dadbcdf1e10e1b74ddc608abd2f98dfb16fbce75277b5232a127f2087ef",
         "5dc9da79a70659a9ad559cb701ded9a2ab9d823aad2f4960cfe370eff4604328",
         ["0ebc5d3437fbe2db158b9f126a1d118e308181031d0a949f8dededebc558ef6a",
          "ca854ea128ed050b41b35ffc1b87b8eb2bde461e9e3b5596ece6b9d5975a0ae0",
          "d37ee418976dd95753c1c73862b9398fa2a2cf9b4ff0fdfe8b30cd95209614b7"]),
        (2, 5,
         "fac54203e7cc696cf0dfcb42c92a1d9dbaf70ad9e621f4bd8d98662f00e3c125",
         "4e3bbb1f7b478dcfe71fb631631519a3bca12c9aefca1612bfce4c13a86264d4",
         ["5f083f0a1a33ca076a95279832580db3e0ef4584bdff1f54c8a360f50de3031e",
          "bc1a0643b12e4d2d7c77918f44e0f4f79a838b6cf9ec5b5c283e1f4d88599e6b"])
        ]

    # Data for leaf inclusion proof test
    sha256_audit_path = [
        "1a208aeebcd1b39fe2de247ee8db9454e1e93a312d206b87f6ca9cc6ec6f1ddd",
        "0a1b78b383f580856f433c01a5741e160d451c185910027f6cc9f828687a40c4",
        "3d1745789bc63f2da15850de1c12a5bf46ed81e1cc90f086148b1662e79aab3d",
        "9095b61e14d8990acf390905621e62b1714fb8e399fbb71de5510e0aef45affe",
        "0a332b91b8fab564e6afd1dd452449e04619b18accc0ff9aa8393cd4928451f2",
        "2336f0181d264aed6d8f3a6507ca14a8d3b3c3a23791ac263e845d208c1ee330",
        "b4ce56e300590500360c146c6452edbede25d4ed83919278749ee5dbe178e048",
        "933f6ddc848ea562e4f9c5cfb5f176941301dad0c6fdb9d1fbbe34fac1be6966",
        "b95a6222958a86f74c030be27c44f57dbe313e5e7c7f4ffb98bcbd3a03bb52f2",
        "daeeb3ce5923defd0faeb8e0c210b753b85b809445d7d3d3cd537a9aabaa9c45",
        "7fadd0a13e9138a2aa6c3fdec4e2275af233b94812784f66bcca9aa8e989f2bc",
        "1864e6ba3e32878610546539734fb5eeae2529991f130c575c73a7e25a2a7c56",
        "12842d1202b1dc6828a17ab253c02e7ce9409b5192430feba44189f39cc02d66",
        "29af64b16fa3053c13d02ac63aa75b23aa468506e44c3a2315edc85d2dc22b11",
        "b527b99934a0bd9edd154e449b0502e2c499bba783f3bc3dfe23364b6b532009",
        "4584db8ae8e351ace08e01f306378a92bfd43611714814f3d834a2842d69faa8",
        "86a9a41573b0d6e4292f01e93243d6cc65b30f06606fc6fa57390e7e90ed580f",
        "a88b98fbe84d4c6aae8db9d1605dfac059d9f03fe0fcb0d5dff1295dacba09e6",
        "06326dc617a6d1f7021dc536026dbfd5fffc6f7c5531d48ef6ccd1ed1569f2a1",
        "f41fe8fdc3a2e4e8345e30216e7ebecffee26ff266eeced208a6c2a3cf08f960",
        "40cf5bde8abb76983f3e98ba97aa36240402975674e120f234b3448911090f8d",
        "b3222dc8658538079883d980d7fdc2bef9285344ea34338968f736b04aeb387a"]

    raw_hex_leaf = (
        "00000000013de9d2b29b000000055b308205573082043fa00302010202072b777b56df"
        "7bc5300d06092a864886f70d01010505003081ca310b30090603550406130255533110"
        "300e060355040813074172697a6f6e61311330110603550407130a53636f7474736461"
        "6c65311a3018060355040a1311476f44616464792e636f6d2c20496e632e3133303106"
        "0355040b132a687474703a2f2f6365727469666963617465732e676f64616464792e63"
        "6f6d2f7265706f7369746f72793130302e06035504031327476f204461646479205365"
        "637572652043657274696669636174696f6e20417574686f726974793111300f060355"
        "040513083037393639323837301e170d3133303131343038353035305a170d31353031"
        "31343038353035305a305331163014060355040a130d7777772e69646e65742e6e6574"
        "3121301f060355040b1318446f6d61696e20436f6e74726f6c2056616c696461746564"
        "311630140603550403130d7777772e69646e65742e6e657430820122300d06092a8648"
        "86f70d01010105000382010f003082010a0282010100d4e4a4b1bbc981c9b8166f0737"
        "c113000aa5370b21ad86a831a379de929db258f056ba0681c50211552b249a02ec00c5"
        "37e014805a5b5f4d09c84fdcdfc49310f4a9f9004245d119ce5461bc5c42fd99694b88"
        "388e035e333ac77a24762d2a97ea15622459cc4adcd37474a11c7cff6239f810120f85"
        "e014d2066a3592be604b310055e84a74c91c6f401cb7f78bdb45636fb0b1516b04c5ee"
        "7b3fa1507865ff885d2ace21cbb28fdaa464efaa1d5faab1c65e4c46d2139175448f54"
        "b5da5aea956719de836ac69cd3a74ca049557cee96f5e09e07ba7e7b4ebf9bf167f4c3"
        "bf8039a4cab4bec068c899e997bca58672bd7686b5c85ea24841e48c46f76830390203"
        "010001a38201b6308201b2300f0603551d130101ff04053003010100301d0603551d25"
        "0416301406082b0601050507030106082b06010505070302300e0603551d0f0101ff04"
        "04030205a030330603551d1f042c302a3028a026a0248622687474703a2f2f63726c2e"
        "676f64616464792e636f6d2f676473312d38332e63726c30530603551d20044c304a30"
        "48060b6086480186fd6d010717013039303706082b06010505070201162b687474703a"
        "2f2f6365727469666963617465732e676f64616464792e636f6d2f7265706f7369746f"
        "72792f30818006082b0601050507010104743072302406082b06010505073001861868"
        "7474703a2f2f6f6373702e676f64616464792e636f6d2f304a06082b06010505073002"
        "863e687474703a2f2f6365727469666963617465732e676f64616464792e636f6d2f72"
        "65706f7369746f72792f67645f696e7465726d6564696174652e637274301f0603551d"
        "23041830168014fdac6132936c45d6e2ee855f9abae7769968cce730230603551d1104"
        "1c301a820d7777772e69646e65742e6e6574820969646e65742e6e6574301d0603551d"
        "0e041604144d3ae8a87ddcf046764021b87e7d8d39ddd18ea0300d06092a864886f70d"
        "01010505000382010100ad651b199f340f043732a71178c0af48e22877b9e5d99a70f5"
        "d78537c31d6516e19669aa6bfdb8b2cc7a145ba7d77b35101f9519e03b58e692732314"
        "1383c3ab45dc219bd5a584a2b6333b6e1bbef5f76e89b3c187ef1d3b853b4910e895a4"
        "57dbe7627e759f56c8484c30b22a74fb00f7b1d7c41533a1fd176cd2a2b06076acd7ca"
        "ddc6ca6d0c2a815f9eb3ef0d03d27e7eebd7824c78fdb51679c03278cfbb2d85ae65a4"
        "7485cb733fc1c7407834f7471ababd68f140983817c6f388b2f2e2bfe9e26608f9924f"
        "16473462d136427d1f2801e4b870b078c20ec4ba21e22ab32a00b76522d523825bcabb"
        "8c7b6142d624be8d2af69ecc36fb5689572a0f59c00000")

    leaf_hash = (
        "7a395c866d5ecdb0cccb623e011dbc392cd348d1d1d72776174e127a24b09c78")
    leaf_index = 848049
    tree_size = 3630887
    expected_root_hash = (
        "78316a05c9bcf14a3a4548f5b854a9adfcd46a4c034401b3ce7eb7ac2f1d0ecb")


    def setUp(self):
        self.verifier = merkle.MerkleVerifier(HexTreeHasher())
        self.STH = namedtuple("STH", ["sha256_root_hash", "tree_size"])
        self.ones = "11" * 32
        self.zeros = "00" * 32

    def test_verify_tree_consistency(self):
        verifier = merkle.MerkleVerifier(HexTreeHasher())
        for test_vector in  MerkleVerifierTest.sha256_proofs:
            self.assertTrue(verifier.verify_tree_consistency(*test_vector))

    def test_verify_tree_consistency_always_accepts_empty_tree(self):
        verifier = merkle.MerkleVerifier(HexTreeHasher())
        # Give some bogus proof too; it should be ignored.
        self.assertTrue(verifier.verify_tree_consistency(
            0, 1,
            "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855",
            "6e340b9cffb37a989ca544e6bb780a2c78901d3fb33738768511a30617afa01d",
            ["6e340b9cffb37a989ca544e6bb780a2c78901d3fb33738768511a30617afa01d"]
            ))

    def test_verify_tree_consistency_for_equal_tree_sizes(self):
        verifier = merkle.MerkleVerifier(HexTreeHasher())
        # Equal tree sizes and hashes, and a bogus proof that should be ignored.
        self.assertTrue(verifier.verify_tree_consistency(
            3, 3,
            "6e340b9cffb37a989ca544e6bb780a2c78901d3fb33738768511a30617afa01d",
            "6e340b9cffb37a989ca544e6bb780a2c78901d3fb33738768511a30617afa01d",
            ["6e340b9cffb37a989ca544e6bb780a2c78901d3fb33738768511a30617afa01d"]
            ))

        # Equal tree sizes but different hashes.
        self.assertRaises(
            error.ConsistencyError, verifier.verify_tree_consistency, 3, 3,
            "6e340b9cffb37a989ca544e6bb780a2c78901d3fb33738768511a30617afa01e",
            "6e340b9cffb37a989ca544e6bb780a2c78901d3fb33738768511a30617afa01d",
            [])

    def test_verify_tree_consistency_newer_tree_is_smaller(self):
        verifier = merkle.MerkleVerifier(HexTreeHasher())
        self.assertRaises(
            ValueError, verifier.verify_tree_consistency, 5, 2,
            "4e3bbb1f7b478dcfe71fb631631519a3bca12c9aefca1612bfce4c13a86264d4",
            "fac54203e7cc696cf0dfcb42c92a1d9dbaf70ad9e621f4bd8d98662f00e3c125",
            ["5f083f0a1a33ca076a95279832580db3e0ef4584bdff1f54c8a360f50de3031e",
             "bc1a0643b12e4d2d7c77918f44e0f4f79a838b6cf9ec5b5c283e1f4d88599e6b"]
            )

    def test_verify_tree_consistency_proof_too_short(self):
        verifier = merkle.MerkleVerifier(HexTreeHasher())
        self.assertRaises(
            error.ProofError, verifier.verify_tree_consistency, 6, 8,
            "76e67dadbcdf1e10e1b74ddc608abd2f98dfb16fbce75277b5232a127f2087ef",
            "5dc9da79a70659a9ad559cb701ded9a2ab9d823aad2f4960cfe370eff4604328",
            ["0ebc5d3437fbe2db158b9f126a1d118e308181031d0a949f8dededebc558ef6a",
             "ca854ea128ed050b41b35ffc1b87b8eb2bde461e9e3b5596ece6b9d5975a0ae0"]
            )

    def test_verify_tree_consistency_bad_second_hash(self):
        verifier = merkle.MerkleVerifier(HexTreeHasher())
        # A bit has been flipped in the second hash.
        self.assertRaises(
            error.ProofError, verifier.verify_tree_consistency, 6, 8,
            "76e67dadbcdf1e10e1b74ddc608abd2f98dfb16fbce75277b5232a127f2087ef",
            "5dc9da79a70659a9ad559cb701ded9a2ab9d823aad2f4960cfe370eff4604329",
            ["0ebc5d3437fbe2db158b9f126a1d118e308181031d0a949f8dededebc558ef6a",
             "ca854ea128ed050b41b35ffc1b87b8eb2bde461e9e3b5596ece6b9d5975a0ae0",
             "d37ee418976dd95753c1c73862b9398fa2a2cf9b4ff0fdfe8b30cd95209614b7"]
            )

    def test_verify_tree_consistency_both_hashes_bad(self):
        verifier = merkle.MerkleVerifier(HexTreeHasher())
        # A bit has been flipped in both hashes.
        self.assertRaises(
            error.ProofError, verifier.verify_tree_consistency, 6, 8,
            "76e67dadbcdf1e10e1b74ddc608abd2f98dfb16fbce75277b5232a127f2087ee",
            "5dc9da79a70659a9ad559cb701ded9a2ab9d823aad2f4960cfe370eff4604329",
            ["0ebc5d3437fbe2db158b9f126a1d118e308181031d0a949f8dededebc558ef6a",
             "ca854ea128ed050b41b35ffc1b87b8eb2bde461e9e3b5596ece6b9d5975a0ae0",
             "d37ee418976dd95753c1c73862b9398fa2a2cf9b4ff0fdfe8b30cd95209614b7"]
            )

    def test_verify_tree_consistency_bad_first_hash(self):
        verifier = merkle.MerkleVerifier(HexTreeHasher())
        # A bit has been flipped in the first hash.
        self.assertRaises(
            error.ConsistencyError, verifier.verify_tree_consistency, 6, 8,
            "76e67dadbcdf1e10e1b74ddc608abd2f98dfb16fbce75277b5232a127f2087ee",
            "5dc9da79a70659a9ad559cb701ded9a2ab9d823aad2f4960cfe370eff4604328",
            ["0ebc5d3437fbe2db158b9f126a1d118e308181031d0a949f8dededebc558ef6a",
             "ca854ea128ed050b41b35ffc1b87b8eb2bde461e9e3b5596ece6b9d5975a0ae0",
             "d37ee418976dd95753c1c73862b9398fa2a2cf9b4ff0fdfe8b30cd95209614b7"]
            )

    def test_calculate_root_hash_good_proof(self):
        verifier = merkle.MerkleVerifier(HexTreeHasher())
        self.assertEqual(
            verifier._calculate_root_hash_from_audit_path(
                self.leaf_hash, self.leaf_index, self.sha256_audit_path[:],
                self.tree_size),
            self.expected_root_hash)

    def test_calculate_root_too_short_proof(self):
        verifier = merkle.MerkleVerifier(HexTreeHasher())
        leaf_index = self.leaf_index + int(
            math.pow(2, len(self.sha256_audit_path) + 1))
        self.assertRaises(
            error.ProofError,
            verifier._calculate_root_hash_from_audit_path,
            self.leaf_hash, leaf_index, self.sha256_audit_path[:],
            self.tree_size)

    def test_verify_leaf_inclusion_good_proof(self):
        verifier = merkle.MerkleVerifier(HexTreeHasher())
        sth = self.STH(self.expected_root_hash, self.tree_size)
        self.assertTrue(
            verifier.verify_leaf_inclusion(
                self.raw_hex_leaf, self.leaf_index, self.sha256_audit_path,
                sth))

    def test_verify_leaf_inclusion_bad_proof(self):
        verifier = merkle.MerkleVerifier(HexTreeHasher())
        # Expect this test to fail by providing an incorrect root hash.
        sth = self.STH(self.zeros, self.tree_size)
        self.assertRaises(
            error.ProofError, verifier.verify_leaf_inclusion,
            self.raw_hex_leaf, self.leaf_index, self.sha256_audit_path, sth)

    def test_verify_leaf_inclusion_incorrect_length_proof(self):
        verifier = merkle.MerkleVerifier(HexTreeHasher())
        sth = self.STH(self.zeros, 4)
        # Too long a proof
        self.assertRaises(
            error.ProofError, verifier.verify_leaf_inclusion,
            self.ones, 0, [self.zeros, self.zeros, self.zeros], sth)
        # Too short a proof
        self.assertRaises(
            error.ProofError, verifier.verify_leaf_inclusion,
            self.ones, 0, [self.zeros], sth)

    def test_verify_leaf_inclusion_single_node_in_tree(self):
        # If there is only one entry in the tree, the tree root hash should be
        # equal to the leaf hash.
        verifier = merkle.MerkleVerifier(HexTreeHasher())
        sth = self.STH(self.leaf_hash, 1)
        self.assertTrue(
            verifier.verify_leaf_inclusion(self.raw_hex_leaf, 0, [], sth))

    def test_verify_leaf_inclusion_rightmost_node_in_tree(self):
        # Show that verify_leaf_inclusion works when required to check a proof
        # for the right-most node: In a tree of 8 nodes, ask for inclusion
        # proof check for leaf 7.
        verifier = merkle.MerkleVerifier(HexTreeHasher())
        hh = HexTreeHasher()
        h_s1 = hh.hash_leaf(self.ones)
        h_c3 = hh.hash_children(self.zeros, h_s1)
        h_c2 = hh.hash_children(self.zeros, h_c3)
        h_root = hh.hash_children(self.zeros, h_c2)
        sth = self.STH(h_root, 8)
        self.assertTrue(
            verifier.verify_leaf_inclusion(
                self.ones, 7, [self.zeros, self.zeros, self.zeros], sth))

    def test_verify_leaf_inclusion_rightmost_node_in_unbalanced_odd_tree(
        self):
        # Show that verify_leaf_inclusion works when required to check a proof
        # for the right-most, even-indexed node: In a tree of 5 nodes, ask for
        # inclusion proof check for leaf 4 (the 5th).
        verifier = merkle.MerkleVerifier(HexTreeHasher())
        hh = HexTreeHasher()
        h_s1 = hh.hash_leaf(self.ones)
        h_root = hh.hash_children(self.zeros, h_s1)
        sth = self.STH(h_root, 5)
        self.assertTrue(
            verifier.verify_leaf_inclusion(self.ones, 4, [self.zeros, ], sth))

    def test_verify_leaf_inclusion_rightmost_node_in_unbalanced_tree_odd_node(
        self):
        # Show that verify_leaf_inclusion works when required to check a proof
        # for the right-most, odd-indexed node: In a tree of 6 nodes, ask for
        # inclusion proof check for leaf 5 (the 6th).
        verifier = merkle.MerkleVerifier(HexTreeHasher())
        hh = HexTreeHasher()
        h_s1 = hh.hash_leaf(self.ones)
        h_l2 = hh.hash_children(self.zeros, h_s1)
        h_root = hh.hash_children(self.zeros, h_l2)
        sth = self.STH(h_root, 6)
        self.assertTrue(
            verifier.verify_leaf_inclusion(
                self.ones, 5, [self.zeros, self.zeros], sth))

    def test_verify_leaf_inclusion_rightmost_node_in_unbalanced_even_tree(
        self):
        # Show that verify_leaf_inclusion works when required to check a proof
        # for the right-most, odd-indexed node: In a tree of 6 nodes, ask for
        # inclusion proof check for leaf 4 (the 5th).
        verifier = merkle.MerkleVerifier(HexTreeHasher())
        hh = HexTreeHasher()
        h_s1 = hh.hash_leaf(self.ones)
        h_l2 = hh.hash_children(h_s1, self.zeros)
        h_root = hh.hash_children(self.zeros, h_l2)
        sth = self.STH(h_root, 6)
        self.assertTrue(
            verifier.verify_leaf_inclusion(
                self.ones, 4, [self.zeros, self.zeros], sth))

    def test_verify_leaf_inclusion_throws_on_bad_indices(self):
        verifier = merkle.MerkleVerifier(HexTreeHasher())
        sth = self.STH("", 6)
        self.assertRaises(ValueError,
            verifier.verify_leaf_inclusion, "", -3, [], sth)
        negative_sth = self.STH("", -3)
        self.assertRaises(ValueError,
            verifier.verify_leaf_inclusion, "", 3, [], negative_sth)

    def test_verify_leaf_inclusion_all_nodes_all_tree_sizes_up_to_4(self):
        leaves = ["aa", "bb", "cc", "dd"]
        hh = HexTreeHasher()
        leaf_hashes = [hh.hash_leaf(l) for l in leaves]
        hc = hh.hash_children
        proofs_per_tree_size = {
            1: [[] ],
            2: [[leaf_hashes[1]], [leaf_hashes[0]]],
            3: [[leaf_hashes[1], leaf_hashes[2]], # leaf 0
                [leaf_hashes[0], leaf_hashes[2]], # leaf 1
                [hc(leaf_hashes[0], leaf_hashes[1])]], # leaf 2
            4: [[leaf_hashes[1], hc(leaf_hashes[2], leaf_hashes[3])], # leaf 0
                [leaf_hashes[0], hc(leaf_hashes[2], leaf_hashes[3])], # leaf 1
                [leaf_hashes[3], hc(leaf_hashes[0], leaf_hashes[1])], # leaf 2
                [leaf_hashes[2], hc(leaf_hashes[0], leaf_hashes[1])], # leaf 3
                ]
            }
        tree = merkle.CompactMerkleTree(hasher=HexTreeHasher())
        verifier = merkle.MerkleVerifier(HexTreeHasher())
        # Increase the tree by one leaf each time
        for i in range(4):
            tree.append(leaves[i])
            tree_size = i + 1
            # ... and check inclusion proof validates for each node
            # of the tree
            for j in range(tree_size):
              proof = proofs_per_tree_size[tree_size][j]
              sth = self.STH(tree.root_hash(), tree_size)
              self.assertTrue(
                  verifier.verify_leaf_inclusion(
                      leaves[j], j, proof, sth))


if __name__ == "__main__":
    unittest.main()



import datetime
import textwrap
import base64

def _write_java_header(output, package):
    year = datetime.date.today().year
    output.write(
        "/*\n"
        " * Copyright (C) %(year)d The Android Open Source Project\n"
        " *\n"
        " * Licensed under the Apache License, Version 2.0 (the \"License\");\n"
        " * you may not use this file except in compliance with the License.\n"
        " * You may obtain a copy of the License at\n"
        " *\n"
        " *      http://www.apache.org/licenses/LICENSE-2.0\n"
        " *\n"
        " * Unless required by applicable law or agreed to in writing, software\n"
        " * distributed under the License is distributed on an \"AS IS\" BASIS,\n"
        " * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n"
        " * See the License for the specific language governing permissions and\n"
        " * limitations under the License.\n"
        " */\n\n"
        "/* This file is generated by print_log_list.py\n"
        " * https://github.com/google/certificate-transparency/blob/master/python/utilities/log_list/print_log_list.py */\n\n"
        "package %(package)s;\n\n" %
        {"year": year,
         "package": package})

def _encode_key(description, key):
    unsigned = (ord(c) for c in key)
    signed = (u - 256 if u > 127 else u for u in unsigned)
    array = textwrap.fill(", ".join("%d" % s for s in signed),
                          width=85,
                          initial_indent="    " * 3,
                          subsequent_indent="    " * 3)

    return ("        // %(description)s\n"
            "        new byte[] {\n"
            "%(array)s\n"
            "        },\n" % {
                "description": description,
                "array": array
            })

def _write_java_class(output, logs, class_name):
    descriptions = ('        "%s",\n' % log["description"] for log in logs)
    urls = ('        "%s",\n' % log["url"] for log in logs)
    keys = (_encode_key(log["description"], base64.decodestring(log["key"]))
            for log in logs)

    output.write(
        "public final class %(class_name)s {\n"
        "    public static final int LOG_COUNT = %(count)d;\n"
        "    public static final String[] LOG_DESCRIPTIONS = new String[] {\n"
        "%(descriptions)s"
        "    };\n"
        "    public static final String[] LOG_URLS = new String[] {\n"
        "%(urls)s"
        "    };\n"
        "    public static final byte[][] LOG_KEYS = new byte[][] {\n"
        "%(keys)s"
        "    };\n"
        "}\n" % {
            "class_name": class_name,
            "count": len(logs),
            "descriptions": "".join(descriptions),
            "urls": "".join(urls),
            "keys": "".join(keys)
        })


def generate_java_source(json_log_list, output_path, class_name):
    with open(output_path, "w") as output:
        logs = json_log_list["logs"]

        [pkg, cls] = class_name.rsplit('.', 1)
        _write_java_header(output, pkg)
        _write_java_class(output, logs, cls)




#!/usr/bin/env python

import unittest

from ct.proto import client_pb2
from ct.proto import test_message_pb2
from ct.serialization import tls_message


valid_test_message = test_message_pb2.TestMessage()
valid_test_message.uint_8 = 0
valid_test_message.uint_16 = 258
valid_test_message.uint_24 = 197637
valid_test_message.uint_32 = 101124105
valid_test_message.uint_48 = 11042563100175
valid_test_message.uint_64 = 255
valid_test_message.fixed_bytes = "\xff\x00"
valid_test_message.var_bytes = "hello"
valid_test_message.var_bytes2 = "world"
valid_test_message.vector_bytes.append("hello")
valid_test_message.vector_bytes.append("world")
valid_test_message.vector_uint32.append(1)
valid_test_message.vector_uint32.append(255)
valid_test_message.test_enum = test_message_pb2.TestMessage.ENUM_1
valid_test_message.select_uint32 = 2
valid_test_message.embedded_message.uint_32 = 3
valid_test_message.repeated_message.add().uint_32 = 4
valid_test_message.repeated_message.add().uint_32 = 256


# Test vectors are given as a list of serialized, hex-encoded components.
serialized_valid_test_message = [
 "00",  # 0: uint_8
 "0102",  # 1: uint_16
 "030405",  # 2: uint_24
 "06070809",  # 3: uint_32
 "0a0b0c0d0e0f",  # 4: uint_48
 "00000000000000ff",  # 5: uint_64
 "ff00",  # 6: fixed_bytes
 "05" + "hello".encode("hex"),  # 7: var_bytes
 "0005" + "world".encode("hex"),  # 8: var_bytes2
 "0c" + "05" + "hello".encode("hex") + "05" +
 "world".encode("hex"),  # 9: vector_bytes
 "0800000001000000ff", # 10: vector_uint32
 "0001", # 11: test_enum
 "00000002", # 12: select_uint32
 "0003",  # 13: embedded_message.uint_32
 "0400040100",  # 14: repeated_message
]


class TLSReaderTest(unittest.TestCase):
    def verify_decode(self, test_vector, test_message):
        serialized = "".join(test_vector).decode("hex")
        message = test_message_pb2.TestMessage()

        tls_message.decode(serialized, message)
        self.assertEqual(test_message, message,
                         msg = "%s vs %s" % (test_message, message))

    def verify_decode_fail(self, test_vector):
        serialized = "".join(test_vector).decode("hex")
        message = test_message_pb2.TestMessage()

        self.assertRaises(tls_message.TLSDecodingError,
                          tls_message.decode, serialized, message)

    def test_decode_valid(self):
        self.verify_decode(serialized_valid_test_message, valid_test_message)
        pass

    def test_decode_valid_select(self):
        test_vector = serialized_valid_test_message[:]
        test_vector[11] = "0000"
        test_vector[12] = ""

        test_message = test_message_pb2.TestMessage()
        test_message.CopyFrom(valid_test_message)
        test_message.test_enum = test_message_pb2.TestMessage.ENUM_0
        test_message.ClearField("select_uint32")
        self.verify_decode(test_vector, test_message)

    def test_decode_invalid_select_fails(self):
        test_vector = serialized_valid_test_message[:]
        test_vector[11] = "0000"

        self.verify_decode_fail(test_vector)

    def test_decode_too_short_fails(self):
        test_vector = serialized_valid_test_message[:]
        # var_bytes2 has a min length of 4
        test_vector[8] = "bit".encode("hex")

        self.verify_decode_fail(test_vector)

    def test_decode_empty(self):
        test_vector = serialized_valid_test_message[:]
        # var_bytes has no min length
        test_vector[7] = "00"

        test_message = test_message_pb2.TestMessage()
        test_message.CopyFrom(valid_test_message)
        test_message.var_bytes = ""
        self.verify_decode(test_vector, test_message)

    def test_decode_too_long_fails(self):
        test_vector = serialized_valid_test_message[:]
        # var_bytes has a max length of 16
        test_vector[7] = "16" + "Iamtoolongformyowngood".encode("hex")

        self.verify_decode_fail(test_vector)

    def test_decode_repeated_too_short_fails(self):
        test_vector = serialized_valid_test_message[:]
        # repeated_uint32 has a min total length of 4
        test_vector[10] = "00"

        self.verify_decode_fail(test_vector)

    def test_decode_repeated_too_long_fails(self):
        test_vector = serialized_valid_test_message[:]
        # repeated_uint32 has a max total length of 8
        test_vector[10] = "0c" + "00"*12

        self.verify_decode_fail(test_vector)

    def test_decode_repeated_invalid_contents_fails(self):
        test_vector = serialized_valid_test_message[:]
        # repeated_uint32 must be a multiple of 4
        test_vector[10] = "02" + "0000"

        self.verify_decode_fail(test_vector)

    def test_read_longer_buffer(self):
        test_vector = serialized_valid_test_message[:]
        test_vector.append("somegarbageintheend".encode("hex"))
        serialized = "".join(test_vector).decode("hex")
        message = test_message_pb2.TestMessage()

        reader = tls_message.TLSReader(serialized)
        reader.read(message)

        self.assertEqual(valid_test_message, message,
                         msg = "%s vs %s" % (valid_test_message, message))
        self.assertFalse(reader.finished())


class TLSWriterTest(unittest.TestCase):
    def verify_encode(self, test_message, test_vector):
        serialized = tls_message.encode(test_message)
        self.assertEqual("".join(test_vector), serialized.encode("hex"))

    def verify_encode_fails(self, test_message):
        self.assertRaises(tls_message.TLSEncodingError,
                          tls_message.encode, test_message)

    def test_encode(self):
        self.verify_encode(valid_test_message, serialized_valid_test_message)

    def test_encode_ignores_skipped_fields(self):
        test_message = test_message_pb2.TestMessage()
        test_message.CopyFrom(valid_test_message)
        test_message.skip_uint32 = 42
        self.verify_encode(test_message, serialized_valid_test_message)

    def test_encode_ignores_bad_select(self):
        test_vector = serialized_valid_test_message[:]
        test_vector[11] = "0000"
        test_vector[12] = ""

        test_message = test_message_pb2.TestMessage()
        test_message.CopyFrom(valid_test_message)
        test_message.test_enum = test_message_pb2.TestMessage.ENUM_0
        self.verify_encode(test_message, test_vector)

    def test_encode_too_large_value_fails(self):
        test_message = test_message_pb2.TestMessage()
        test_message.CopyFrom(valid_test_message)
        test_message.uint_8 = 65000
        self.verify_encode_fails(test_message)

    def test_encode_bad_length_fails(self):
        test_message = test_message_pb2.TestMessage()
        test_message.CopyFrom(valid_test_message)
        test_message.fixed_bytes = "hello"
        self.verify_encode_fails(test_message)

    def test_encode_too_short_fails(self):
        test_message = test_message_pb2.TestMessage()
        test_message.CopyFrom(valid_test_message)
        test_message.var_bytes2 = "sho"
        self.verify_encode_fails(test_message)

    def test_encode_too_long_fails(self):
        test_message = test_message_pb2.TestMessage()
        test_message.CopyFrom(valid_test_message)
        test_message.var_bytes = "Iamtoolongformyowngood"
        self.verify_encode_fails(test_message)

    def test_encode_repeated_too_long_fails(self):
        test_message = test_message_pb2.TestMessage()
        test_message.CopyFrom(valid_test_message)
        test_message.vector_uint32.extend([1, 2, 3, 4])
        self.verify_encode_fails(test_message)

    def test_encode_repeated_too_short_fails(self):
        test_message = test_message_pb2.TestMessage()
        test_message.CopyFrom(valid_test_message)
        test_message.ClearField("vector_uint32")
        self.verify_encode_fails(test_message)


class SCTEncodingTest(unittest.TestCase):
    def setUp(self):
        sct_proto = client_pb2.SignedCertificateTimestamp()
        sct_proto.version = client_pb2.V1
        sct_proto.id.key_id = (
            "a4b90990b418581487bb13a2cc67700a3c359804f91bdfb8e377cd0ec80ddc10"
            ).decode('hex')

        sct_proto.timestamp = 1365427532443
        sct_proto.signature.hash_algorithm = client_pb2.DigitallySigned.SHA256
        sct_proto.signature.sig_algorithm = client_pb2.DigitallySigned.ECDSA
        sct_proto.signature.signature = (
            "304502210089de897f603e590b1aa0d7c4236c2f697e90602795f7a469215fda5e"
            "460123fc022065ab501ce3dbaf49bd563d1c9ff0ac76120bc11f65a44122b3cd8b"
            "89fc77a48c").decode("hex")
        self._sct_proto = sct_proto

    def test_correctly_encodes_sct(self):
        sct = tls_message.encode(self._sct_proto)
        expected_sct = ("00a4b90990b418581487bb13a2cc67700a3c359804f91bdfb8e377"
                        "cd0ec80ddc100000013de9d2b29b000004030047304502210089de"
                        "897f603e590b1aa0d7c4236c2f697e90602795f7a469215fda5e46"
                        "0123fc022065ab501ce3dbaf49bd563d1c9ff0ac76120bc11f65a4"
                        "4122b3cd8b89fc77a48c").decode("hex")
        self.assertEqual(sct, expected_sct)

    def test_correctly_encodes_sct_list_one_sct(self):
        # Taken from the C++ serializer test, to ensure this encoder
        # produces results compatible with the C++ one.
        single_sct = ("0069616d617075626c69636b657973686174776f6669766573697864"
                      "696765737400000000000004d20000040300097369676e6174757265"
                      ).decode("hex")
        sct_list = client_pb2.SignedCertificateTimestampList()
        sct_list.sct_list.append(single_sct)
        encoded_sct_list = tls_message.encode(sct_list)
        self.assertEqual(encoded_sct_list[:4],
                         "003a0038".decode("hex"))
        self.assertEqual(encoded_sct_list[4:], single_sct)

    def test_correctly_encodes_sct_list_multiple_scts(self):
        first_sct = tls_message.encode(self._sct_proto)
        sct_proto_2 = client_pb2.SignedCertificateTimestamp()
        sct_proto_2.CopyFrom(self._sct_proto)
        sct_proto_2.timestamp = 1365427530000
        second_sct = tls_message.encode(sct_proto_2)

        sct_list = client_pb2.SignedCertificateTimestampList()
        sct_list.sct_list.extend([first_sct, second_sct])
        encoded_sct_list = tls_message.encode(sct_list)
        # First 2 bytes are list length prefix - 240 bytes in total
        # Next 2 bytes are the length of the first SCT: 118
        self.assertEqual(encoded_sct_list[:4],
                         "00f00076".decode("hex"))
        first_sct_end = len(first_sct) + 4
        # The actual SCT
        self.assertEqual(encoded_sct_list[4:first_sct_end], first_sct)
        # Next 2 bytes are the length of the second SCT (118 again)
        self.assertEqual(encoded_sct_list[first_sct_end:first_sct_end+2],
                         "0076".decode("hex"))
        # The 2nd SCT
        self.assertEqual(encoded_sct_list[first_sct_end+2:], second_sct)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env trial
import gflags
import json
import mock
import sys
import urlparse

from ct.client import log_client
from ct.client import async_log_client
from ct.client import log_client_test_util as test_util
from ct.client.db import database
from twisted.internet import defer
from twisted.internet import task
from twisted.internet import reactor
from twisted.python import failure
from twisted.test import proto_helpers
from twisted.trial import unittest

FLAGS = gflags.FLAGS


class ResponseBodyHandlerTest(unittest.TestCase):
    def test_send(self):
        finished = defer.Deferred()
        handler = async_log_client.ResponseBodyHandler(finished)
        transport = proto_helpers.StringTransportWithDisconnection()
        handler.makeConnection(transport)
        transport.protocol = handler
        handler.dataReceived("test")
        transport.loseConnection()
        finished.addCallback(self.assertEqual, "test")
        return finished

    def test_send_chunks(self):
        test_msg = "x"*1024
        chunk_size = 100
        finished = defer.Deferred()
        handler = async_log_client.ResponseBodyHandler(finished)
        transport = proto_helpers.StringTransportWithDisconnection()
        handler.makeConnection(transport)
        transport.protocol = handler
        sent = 0
        while sent < len(test_msg):
            handler.dataReceived(test_msg[sent:sent + chunk_size])
            sent += chunk_size
        transport.loseConnection()
        finished.addCallback(self.assertEqual, test_msg)
        return finished

    def test_buffer_overflow(self):
        original = FLAGS.response_buffer_size_bytes
        FLAGS.response_buffer_size_bytes = 10
        test_msg = "x"*11
        finished = defer.Deferred()
        handler = async_log_client.ResponseBodyHandler(finished)
        transport = proto_helpers.StringTransportWithDisconnection()
        handler.makeConnection(transport)
        transport.protocol = handler
        handler.dataReceived(test_msg)
        transport.loseConnection()
        # TODO(ekasper): find a more elegant and robust way to save flags.
        FLAGS.response_buffer_size_bytes = original
        return self.assertFailure(finished,
                                  async_log_client.HTTPResponseSizeExceededError)


class AsyncLogClientTest(unittest.TestCase):
    class FakeHandler(test_util.FakeHandlerBase):

        # A class that mimics twisted.web.iweb.IResponse. Note: the IResponse
        # interface is only partially implemented.
        class FakeResponse(object):
            def __init__(self, code, reason, json_content=None):
                self.code = code
                self.phrase = reason
                self.headers = AsyncLogClientTest.FakeHandler.FakeHeader()
                if json_content is not None:
                    self._body = json.dumps(json_content)
                else:
                    self._body = ""

            def deliverBody(self, protocol):
                transport = proto_helpers.StringTransportWithDisconnection()
                protocol.makeConnection(transport)
                transport.protocol = protocol
                protocol.dataReceived(self._body)
                transport.loseConnection()

        @classmethod
        def make_response(cls, code, reason, json_content=None):
            return cls.FakeResponse(code, reason, json_content=json_content)

        class FakeHeader(object):
            def getAllRawHeaders(self):
                return []

    # Twisted doesn't (yet) have an official fake Agent:
    # https://twistedmatrix.com/trac/ticket/4024
    class FakeAgent(object):
        def __init__(self, responder):
            self._responder = responder

        def request(self, method, uri):
            if method != "GET":
                return defer.fail(failure.Failure())
            # Naive, for testing.
            path, _, params = uri.partition("?")
            params = urlparse.parse_qs(params)
            # Take the first value of each parameter.
            if any([len(params[key]) != 1 for key in params]):
                return defer.fail(failure.Failure())
            params = {key: params[key][0] for key in params}
            response = self._responder.get_response(path, params=params)
            return defer.succeed(response)

    class FakeDB(object):
        def scan_entries(self, first, last):
            raise database.KeyError("boom!")

        def store_entries(self, entries):
            self.entries = list(entries)

    def setUp(self):
        self.clock = task.Clock()

    def one_shot_client(self, json_content):
        """Make a one-shot client and give it a mock response."""
        mock_handler = mock.Mock()
        response = self.FakeHandler.make_response(200, "OK",
                                                  json_content=json_content)
        mock_handler.get_response.return_value = response
        return async_log_client.AsyncLogClient(self.FakeAgent(mock_handler),
                                         test_util.DEFAULT_URI,
                                         reactor=self.clock)

    def default_client(self, entries_db=None, reactor_=None):
        # A client whose responder is configured to answer queries for the
        # correct uri.
        if reactor_ is None:
            reactor_ = self.clock
        return async_log_client.AsyncLogClient(self.FakeAgent(
            self.FakeHandler(test_util.DEFAULT_URI)), test_util.DEFAULT_URI,
                                         entries_db=entries_db,
                                         reactor=reactor_)

    def test_get_sth(self):
        client = self.default_client()
        self.assertEqual(test_util.DEFAULT_STH,
                         self.successResultOf(client.get_sth()))

    def test_get_sth_raises_on_invalid_response(self):
        json_sth = test_util.sth_to_json(test_util.DEFAULT_STH)
        json_sth.pop("timestamp")
        client = self.one_shot_client(json_sth)
        return self.assertFailure(client.get_sth(),
                                  log_client.InvalidResponseError)

    def test_get_sth_raises_on_invalid_base64(self):
        json_sth = test_util.sth_to_json(test_util.DEFAULT_STH)
        json_sth["tree_head_signature"] = "garbagebase64^^^"
        client = self.one_shot_client(json_sth)
        return self.assertFailure(client.get_sth(),
                                  log_client.InvalidResponseError)

    class EntryConsumer(object):
        def __init__(self):
            self.received = []
            self.consumed = defer.Deferred()

        def done(self, result):
            self.result = result
            self.consumed.callback("Done")

        def consume(self, entries):
            self.received += entries
            d = defer.Deferred()
            d.callback(None)
            return d

    # Helper method.
    def get_entries(self, client, start, end, batch_size=0):
        producer = client.get_entries(start, end, batch_size=batch_size)
        consumer = self.EntryConsumer()
        d = producer.startProducing(consumer)
        d.addBoth(consumer.done)
        # Ensure the tasks scheduled in the reactor are invoked.
        # Since start of get entries is delayed, we have to pump to make up for
        # that delay. If some test is going to force get_entries to do more than
        # one fetch, then that test has to take care of additional pumping.
        self.pump_get_entries()
        return consumer

    def pump_get_entries(self,
                     delay=None,
                     pumps=1):
        if not delay:
            delay = FLAGS.get_entries_retry_delay
        # Helper method which advances time past get_entries delay
        for _ in range(0, pumps):
            self.clock.pump([0, delay])

    def test_get_entries(self):
        client = self.default_client()
        consumer = self.get_entries(client, 0, 9)
        self.assertEqual(10, consumer.result)
        self.assertTrue(test_util.verify_entries(consumer.received, 0, 9))

    def test_get_sth_consistency(self):
        client = self.default_client()
        self.assertEqual([],
                         self.successResultOf(client.get_sth_consistency(0, 9)))

    def test_get_entries_raises_on_invalid_response(self):
        json_entries = test_util.entries_to_json(test_util.make_entries(0, 9))
        json_entries["entries"][5]["leaf_input"] = "garbagebase64^^^"

        client = self.one_shot_client(json_entries)
        producer = client.get_entries(0, 9)
        # remove exponential back-off
        producer._calculate_retry_delay = lambda _: 1
        consumer = self.EntryConsumer()
        d = producer.startProducing(consumer)
        d.addBoth(consumer.done)
        # pump through retries (with retries there are 2 delays per request and
        # and initial delay)
        self.pump_get_entries(1, FLAGS.get_entries_max_retries * 2 + 1)
        self.assertTrue(consumer.result.check(log_client.InvalidResponseError))
        # The entire response should be discarded upon error.
        self.assertFalse(consumer.received)

    def test_get_entries_raises_on_too_large_response(self):
        large_response = test_util.entries_to_json(
            test_util.make_entries(4, 5))

        client = self.one_shot_client(large_response)
        producer = client.get_entries(4, 4)
        # remove exponential back-off
        producer._calculate_retry_delay = lambda _: 1
        consumer = self.EntryConsumer()
        d = producer.startProducing(consumer)
        d.addBoth(consumer.done)
        # pump through retries (with retries there are 2 delays per request and
        # initial delay)
        self.pump_get_entries(1, FLAGS.get_entries_max_retries * 2 + 1)
        self.assertTrue(consumer.result.check(log_client.InvalidResponseError))

    def test_get_entries_succedes_after_retry(self):
        json_entries = test_util.entries_to_json(test_util.make_entries(0, 9))
        json_entries["entries"][5]["leaf_input"] = "garbagebase64^^^"
        client = self.one_shot_client(json_entries)
        producer = client.get_entries(0, 9)
        # remove exponential back-off
        producer._calculate_retry_delay = lambda _: 1
        consumer = self.EntryConsumer()
        d = producer.startProducing(consumer)
        d.addBoth(consumer.done)
        # pump retries halfway through (there are actually two delays before
        # firing requests, so this loop will go only through half of retries)
        self.pump_get_entries(1, FLAGS.get_entries_max_retries)
        self.assertFalse(hasattr(consumer, 'result'))
        json_entries = test_util.entries_to_json(test_util.make_entries(0, 9))
        response = self.FakeHandler.make_response(200, "OK",
                                                  json_content=json_entries)
        client._handler._agent._responder.get_response.return_value = response
        self.pump_get_entries(1)
        self.assertTrue(test_util.verify_entries(consumer.received, 0, 9))

    def test_get_entries_raises_if_query_is_larger_than_tree_size(self):
        client = async_log_client.AsyncLogClient(
            self.FakeAgent(self.FakeHandler(
                test_util.DEFAULT_URI, tree_size=3)), test_util.DEFAULT_URI,
            reactor=self.clock)
        consumer = self.get_entries(client, 0, 9)
        # also pump error
        self.pump_get_entries()
        self.assertTrue(consumer.result.check(log_client.HTTPClientError))

    def test_get_entries_returns_all_in_batches(self):
        mock_handler = mock.Mock()
        fake_responder = self.FakeHandler(test_util.DEFAULT_URI)
        mock_handler.get_response.side_effect = (
            fake_responder.get_response)

        client = async_log_client.AsyncLogClient(self.FakeAgent(mock_handler),
                                           test_util.DEFAULT_URI,
                                           reactor=self.clock)
        consumer = self.get_entries(client, 0, 9, batch_size=4)
        self.assertEqual(10, consumer.result)
        self.assertTrue(test_util.verify_entries(consumer.received, 0, 9))
        self.assertEqual(3, len(mock_handler.get_response.call_args_list))

    def test_get_entries_returns_all_for_limiting_server(self):
        client = async_log_client.AsyncLogClient(
            self.FakeAgent(
                self.FakeHandler(test_util.DEFAULT_URI, entry_limit=3)),
            test_util.DEFAULT_URI, reactor=self.clock)
        consumer = self.get_entries(client, 0, 9)
        # 1 pump in get_entries and 3 more so we fetch everything
        self.pump_get_entries(pumps=3)
        self.assertTrue(test_util.verify_entries(consumer.received, 0, 9))

    class PausingConsumer(object):
        def __init__(self, pause_at):
            self.received = []
            self.pause_at = pause_at
            self.already_paused = False
            self.result = None

        def registerProducer(self, producer):
            self.producer = producer

        def done(self, result):
            self.result = result

        def consume(self, entries):
            self.received += entries
            if (not self.already_paused and
                len(self.received) >= self.pause_at):
                self.producer.pauseProducing()
                self.already_paused = True
            d = defer.Deferred()
            d.callback(None)
            return d

    def test_get_entries_pause_resume(self):
        client = self.default_client()
        producer = client.get_entries(0, 9, batch_size=4)
        consumer = self.PausingConsumer(4)
        consumer.registerProducer(producer)
        d = producer.startProducing(consumer)
        d.addBoth(consumer.done)
        # fire all pending callbacks, and then fire request
        self.pump_get_entries()
        self.assertTrue(test_util.verify_entries(consumer.received, 0, 3))
        self.assertEqual(4, len(consumer.received))
        self.assertIsNone(consumer.result)
        producer.resumeProducing()
        # pump next 2 batches
        self.pump_get_entries(pumps=2)
        self.assertEqual(10, consumer.result)
        self.assertTrue(test_util.verify_entries(consumer.received, 0, 9))

    def test_get_entries_use_stored_entries(self):
        fake_db = self.FakeDB()
        # if client tries to fetch entries instead of taking them from db, then
        # he will get 0 - 9 entries. If he uses db then he will get 10 - 19
        fake_db.scan_entries = mock.Mock(
                return_value=test_util.make_entries(10, 19))
        client = self.default_client(entries_db=fake_db, reactor_=reactor)
        consumer = self.get_entries(client, 0, 9)
        consumer.consumed.addCallback(lambda _:
                                  self.assertEqual(len(consumer.received), 10))
        consumer.consumed.addCallback(lambda _:
            [self.assertEqual(test_util.make_entry(i + 10), consumer.received[i])
             for i in range(0, 9)])

    def test_get_entries_tries_to_fetch_if_not_available_in_db(self):
        fake_db = self.FakeDB()
        fake_db.scan_entries = mock.Mock(return_value=None)
        client = self.default_client(entries_db=fake_db)
        consumer = self.get_entries(client, 0, 9)
        test_util.verify_entries(consumer.received, 0, 9)

    def test_get_entries_stores_entries(self):
        fake_db = self.FakeDB()
        client = self.default_client(entries_db=fake_db, reactor_=reactor)
        consumer = self.get_entries(client, 0, 9)
        consumer.consumed.addCallback(lambda _:
                            test_util.verify_entries(consumer.received, 0, 9))
        consumer.consumed.addCallback(lambda _:
                            test_util.verify_entries(fake_db.entries, 0, 9))
        return consumer.consumed

    class BadEntryConsumer(EntryConsumer):
        def consume(self, entries):
            self.received += entries
            d = defer.Deferred()
            d.errback(ValueError("Boom!"))
            return d

    def test_get_entries_fires_done_if_consumer_raises(self):
        client = self.default_client()
        producer = client.get_entries(0, 9)
        consumer = self.BadEntryConsumer()
        d = producer.startProducing(consumer)
        d.addBoth(consumer.done)
        self.pump_get_entries()
        self.assertTrue(consumer.result.check(ValueError))

if __name__ == "__main__" or __name__ == "ct.client.async_log_client_test":
    sys.argv = FLAGS(sys.argv)



#!/usr/bin/env python

# Given a certificate chain that the log won't accept, try to fix it up
# into one that will be accepted.

# Based on pyasn1 example code.

from base64 import b64encode
from ct.crypto.pem import PemError
from ct.crypto.pem import from_pem
from pyasn1 import debug
# Why doesn't this work?
#from pyasn1.codec.ber import stDumpRawValue
from pyasn1.codec.der import decoder
from pyasn1.codec.der import encoder
from pyasn1.error import PyAsn1Error
from pyasn1.type import namedtype
from pyasn1.type import univ
from pyasn1_modules import pem
from pyasn1_modules import rfc2459
from pyasn1_modules import rfc2315
import sys
from urllib2 import urlopen

if len(sys.argv) != 2:
  print """Usage:
  $ %s somecertificates.pem""" % sys.argv[0]
  sys.exit(-1)

cStart = '-----BEGIN CERTIFICATE-----'
cEnd = '-----END CERTIFICATE-----'

certType = rfc2459.Certificate()

# RFC 2459 is not sufficient for X509v3 certificates, extra stuff here.
# RFC 5280 4.2.2.1

id_pe_authorityInfoAccess = univ.ObjectIdentifier('1.3.6.1.5.5.7.1.1')

class AccessDescription(univ.Sequence):
  """
     AccessDescription  ::=  SEQUENCE {
                accessMethod          OBJECT IDENTIFIER,
                accessLocation        GeneralName  }
  """
  componentType = namedtype.NamedTypes(
    namedtype.NamedType('accessMethod', univ.ObjectIdentifier()),
    namedtype.NamedType('accessLocation', rfc2459.GeneralName()))

class AuthorityInfoAccessSyntax(univ.SequenceOf):
  """
  AuthorityInfoAccessSyntax  ::=
             SEQUENCE SIZE (1..MAX) OF AccessDescription
  """
  # FIXME: SIZE not encoded.
  componentType = AccessDescription()

id_ad_caIssuers = univ.ObjectIdentifier('1.3.6.1.5.5.7.48.2')

# End of RFC 5280 4.2.2.1

def getIssuersFromAIA(cert):
  tbs = cert.getComponentByName('tbsCertificate')
  extensions = tbs.getComponentByName('extensions') or []

  allIssuers = []
  for extension in extensions:
    oid = extension.getComponentByName('extnID')
    if oid != id_pe_authorityInfoAccess:
      continue
    
    print extension.prettyPrint()

    value, rest = decoder.decode(extension.getComponentByName('extnValue'),
                                 asn1Spec=univ.OctetString())
    assert rest == ""
    aia, rest = decoder.decode(value, asn1Spec=AuthorityInfoAccessSyntax())
    assert rest == ""

    print aia.prettyPrint()

    for ad in aia:
      oid = ad.getComponentByName('accessMethod')
      if oid != id_ad_caIssuers:
        continue
      
      print ad.prettyPrint()

      loc = ad.getComponentByName('accessLocation').\
        getComponentByName('uniformResourceIdentifier')
      print type(loc), loc

      certHandle = urlopen(str(loc))
      # RFC 5280 says this should either be 'application/pkix-cert' or
      # 'application/pkcs7-mime' (in which case the result should be a
      # "certs-only" PCKS#7 response, as specified in RFC 2797). Of
      # course, we see other values, so just try both formats.
      print certHandle.info().gettype()
      issuer = certHandle.read()

      # Have we got an (incorrect, but let's fix it) PEM encoded cert?
      if issuer.startswith('-----'):
        try:
          (issuer, _) = from_pem(issuer, ['CERTIFICATE'])
        except PemError as e:
          print "PEM decode failed:", e
          print "For cert:", issuer

      # Is it a certificate?
      try:
        cert, rest = decoder.decode(issuer, asn1Spec=certType)
        assert rest == ""
        allIssuers.append(cert)
        continue
      except PyAsn1Error as e:
        # On failure, try the next thing
        print "Cert decode failed:", e
        pass

      # If not, it had better be PKCS#7 "certs-only"
      try:
        pkcs7, rest = decoder.decode(issuer, asn1Spec=rfc2315.ContentInfo())
        assert rest == ""
        assert pkcs7.getComponentByName('contentType') == rfc2315.signedData
        signedData = decoder.decode(pkcs7.getComponentByName('content'),
                                    asn1Spec=rfc2315.SignedData())
      except PyAsn1Error as e:
        # Give up
        print "PKCS#7 decode also failed:", e
        print "Skipping issuer URL:", loc
        continue

      for signedDatum in signedData:
        # FIXME: why does this happen? Example is at
        # http://crt.usertrust.com/AddTrustExternalCARoot.p7c.
        if signedDatum == '':
          print "** Skipping strange Any('') in PKCS7 **"
          continue
        certs = signedDatum.getComponentByName('certificates')
        for c in certs:
          cert = c.getComponentByName('certificate')
          allIssuers.append(cert)
  return allIssuers

# Note that this is a non-standard encoding of the DN, but unlike the
# standard encoding it captures nesting information. That is,
# attributes that are within a single RelativeDistinguishedName are
# surrounded by [].
def DNToString(dn):
  rdns = dn.getComponent()
  ret = ''
  for rdn in rdns:
    ret += '['
    
    for attr in rdn:
      attrType = attr.getComponentByName('type')

      if attrType == rfc2459.emailAddress:
        val, rest = decoder.decode(attr.getComponentByName('value'),
                                   asn1Spec=rfc2459.Pkcs9email())
        assert rest == ""

        # Strictly speaking, this is IA5, not ASCII.
        val = str(val).decode('ascii')
      else:
        val, rest = decoder.decode(attr.getComponentByName('value'),
                                   asn1Spec=rfc2459.X520name())
        assert rest == ""
      
        valt = val.getName()
        val = val.getComponent()
      
        if valt == 'printableString':
          val = str(val)
        elif valt == 'teletexString':
          # Strictly this is a T.61 string. T.61 no longer exists as a
          # standard and some certs mark ISO 8859-1 as
          # teletexString. And we should never see this, but we do.
          val = str(val).decode('iso8859-1')
        elif valt == 'utf8String':
          val = str(val)
        else:
          print valt
          assert False
        
      assert val is not None
      
      ret += '/' + str(attrType) + '=' + val
      
    ret += ']'
    
  return ret

certs = {}
inChain = []

certfile = open(sys.argv[1])

while 1:
  idx, substrate = pem.readPemBlocksFromFile(certfile, (cStart, cEnd))
  if not substrate:
    break

  cert, rest = decoder.decode(substrate, asn1Spec=certType)
  assert rest == ""

  tbs = cert.getComponentByName('tbsCertificate')

  subjectDN = tbs.getComponentByName('subject')
  print DNToString(subjectDN)

  certs[DNToString(subjectDN)] = cert
  inChain.append(cert)

#for subject, cert in certs.iteritems():
#  print subject

# Assume the first cert in the chain is the final cert
outChain = [inChain[0]]

while True:
  assert len(outChain) < 100
  
  cert = outChain[-1]

  tbs = cert.getComponentByName('tbsCertificate')

  subjectDN = tbs.getComponentByName('subject')
  print 'subject:', DNToString(subjectDN)

  issuerDN = tbs.getComponentByName('issuer')
  #print issuerDN.prettyPrint()
  issuerDNstr = DNToString(issuerDN)
  print 'issuer:', issuerDNstr

  print

  if issuerDN == subjectDN:
    break

  if issuerDNstr in certs:
    issuer = certs[issuerDNstr]
  else:
    issuers = getIssuersFromAIA(cert)
    if len(issuers) == 0:
      print "Can't get issuer, giving up"
      break

    issuer = None
    for i in issuers:
      tbs = i.getComponentByName('tbsCertificate')
      subjectDN = tbs.getComponentByName('subject')
      print 'issuer subject:', DNToString(subjectDN)
      if subjectDN == issuerDN:
        issuer = i
        break

  assert issuer is not None

  outChain.append(issuer)

if len(outChain) == 1:
  tbs = outChain[0].getComponentByName('tbsCertificate')

  subjectDN = tbs.getComponentByName('subject')
  issuerDN = tbs.getComponentByName('issuer')
  if subjectDN == issuerDN:
    print "Chain consists of 1 self-signed certificate"
    exit(1)

for cert in outChain:
    print cStart
    b64 = b64encode(encoder.encode(cert))
    for n in range(0, len(b64), 64):
      print b64[n:n+64]
    print cEnd

print('*** %d PEM cert(s) deserialized, fixed chain is %d long' % (
  len(inChain),
  len(outChain)))



#!/usr/bin/env python
import unittest

import mock
import sys
from ct.client import db_reporter
import gflags


class DbReporterTest(unittest.TestCase):
    def test_report(self):
        db = mock.MagicMock()
        reporter = db_reporter.CertDBCertificateReport(db, 1)
        for j in range(1, 6):
            for i in range(0, 10):
                reporter._batch_scanned_callback([(None, None)])
            reporter.report()
            self.assertEqual(db.store_certs_desc.call_count, 10 * j)

    def test_db_raising_does_not_stall_reporter(self):
        db = mock.Mock()
        db.store_certs_desc.side_effect = [ValueError("Boom!"), None]

        reporter = db_reporter.CertDBCertificateReport(db, 1)
        reporter._batch_scanned_callback([(None, None)])
        reporter._batch_scanned_callback([(None, None)])
        reporter.report()
        self.assertEqual(db.store_certs_desc.call_count, 2)

if __name__ == '__main__':
    sys.argv = gflags.FLAGS(sys.argv)
    unittest.main()



import hashlib, base64

# Basic verifiable log, for demo purposes only
class VerifiableLog:
  def __init__(self):
    self._entries = []
    self._cache = {}

  def append(self, val):
    self._entries.append(val)

  def get_entries(self, start, end):
    return self._entries[start:end + 1]

  def get_tree_head(self, seq=None):
    if seq is None:
      seq = len(self._entries)
    if seq > len(self._entries):
      raise
    return {
      'tree_size': seq,
      'sha256_root_hash': base64.b64encode(self._calc_mth(0, seq)) if seq else None,
    }

  def consistency_proof(self, first, second):
    return [base64.b64encode(self._calc_mth(a, b)) for a, b in self._subproof(first, 0, second, True)]

  def inclusion_proof(self, m, n):
    return [base64.b64encode(self._calc_mth(a, b)) for a, b in self._path(m, 0, n)]

  def _calc_mth(self, start, end):
    k = '%i-%i' % (start, end)
    rv = self._cache.get(k, None)
    if not rv:
      stack = []
      tree_size = end - start
      for idx, leaf in enumerate(self._entries[start:end]):
        stack.append(hashlib.sha256(chr(0) + leaf).digest())
        for _ in range(bin(idx).replace('b', '')[::-1].index('0') if idx + 1 < tree_size else len(stack) - 1):
          stack[-2:] = [hashlib.sha256(chr(1) + stack[-2] + stack[-1]).digest()]
      rv = stack[0]
      self._cache[k] = rv
    return rv

  def _subproof(self, m, start_n, end_n, b):
    n = end_n - start_n
    if m == n:
      if b:
        return []
      else:
        return [(start_n, end_n)]
    else:
      k = 1 << (len(bin(n - 1)) - 3)
      if m <= k:
        return self._subproof(m, start_n, start_n + k, b) + [(start_n + k, end_n)]
      else:
        return self._subproof(m - k, start_n + k, end_n, False) + [(start_n, start_n + k)]

  def _path(self, m, start_n, end_n):
    n = end_n - start_n
    if n == 1:
      return []
    else:
      k = 1 << (len(bin(n - 1)) - 3)
      if m < k:
        return self._path(m, start_n, start_n + k) + [(start_n + k, end_n)]
      else:
        return self._path(m - k, start_n + k, end_n) + [(start_n, start_n + k)]



import abc

class LogDB(object):
    """Database interface for storing client-side CT data."""
    __metaclass__ = abc.ABCMeta

    # The largest BSON can handle
    timestamp_max = 2**63-1

    @abc.abstractmethod
    def add_log(self, metadata):
        """Store log metadata. This creates the necessary mappings between
        tables so all logs must be explicitly added.
        Params:
            metadata: a client_pb2.CtLogMetadata proto."""

    @abc.abstractmethod
    def update_log(self, metadata):
        """Add a new log or update existing log metadata. When updating, does
        not verify that the new metadata is consistent with stored values.
        Params:
            metadata: a client_pb2.CtLogMetadata proto."""

    @abc.abstractmethod
    def logs(self):
        """A generator that yields all currently known logs."""

    @abc.abstractmethod
    def store_sth(self, log_server, audited_sth):
        """Store the STH in the database.
        Will store the STH with a unique ID unless an exact copy already exists.
        Params:
            log_server: the server name, i.e., the <log_server> path prefix
            audited_sth: a client_pb2.AuditedSth proto
        """

    @abc.abstractmethod
    def get_latest_sth(self, log_server):
        """"Get the AuditedSth with the latest timestamp."""

    @abc.abstractmethod
    def scan_latest_sth_range(self, log_server, start=0, end=timestamp_max,
                              limit=0):
        """Scan STHs by timestamp
        Args:
            logid: CT log to scan
            start: earliest timestamp
            end: latest timestamp
            limit: maximum number of entries to return. Default is no limit.
        Yields:
            the AuditedSth protos in descending order of timestamps
        Note the scan may be keeping the database connection open until the
        generator is exhausted.
        """

    @abc.abstractmethod
    def get_log_id(self, log_server):
        """Get id in database of log_server."""



import logging
import sqlite3

from ct.client.db import temp_db
from ct.client.db import database
from ct.client.db import sqlite_connection as sqlitecon
from ct.proto import client_pb2

class SQLiteTempDBFactory(object):
    """A database factory that manages mappings from public identifiers
    (log names) to SQLite databases."""
    def __init__(self, connection_manager, database_dir):
        """Initialize the database factory.
        Args:
            connection_manager: an SQLiteConnectionManager object
            database_dir: the directory where the database files reside.
        """
        self.__mgr = connection_manager
        self.__database_dir = database_dir
        # This is the meta-table mapping database IDs to server names.
        with self.__mgr.get_connection() as conn:
            conn.execute("CREATE TABLE IF NOT EXISTS database_mapping("
                         "id INTEGER PRIMARY KEY, server_name TEXT UNIQUE)")
        self.__tables = ["database_mapping"]

    def __repr__(self):
        return "%r(%r)" % (self.__class__.__name__, self.__mgr)

    def __str__(self):
        return "%s(%s, tables: %s): " % (self.__class__.__name__,
                                         self.__mgr, self.__tables)

    @staticmethod
    def __database_id_to_name(database_id):
        return "db%d" % database_id

    def __get_db_name(self, cursor, log_server):
        cursor.execute("SELECT id from database_mapping WHERE server_name = ?",
                       (log_server,))
        row = cursor.fetchone()
        if row is None:
            raise database.KeyError("No database for log server %s" %
                                    log_server)
        return self.__database_id_to_name(row["id"])

    # TODO(ekasper): add a remove_storage() for removing obsolete data.
    def create_storage(self, log_server):
        """Create a SQLiteTempDB object pointing to the temporary storage of a
        given log server. If the temporary storage does not exist, creates one.
        Args:
        log_server: the server name.
        """
        with self.__mgr.get_connection() as conn:
            cursor = conn.cursor()
            try:
                database_name = self.__get_db_name(cursor, log_server)
            except database.KeyError:
                try:
                    cursor.execute("INSERT INTO database_mapping(server_name) "
                                   "VALUES (?)", (log_server,))
                    database_name = self.__database_id_to_name(cursor.lastrowid)
                except sqlite3.IntegrityError as e:
                    raise database.KeyError("Failed to create a table mapping "
                                            "for server %s: is a concurrent "
                                            "factory running?\n%s" %
                                            (log_server, e))
        return SQLiteTempDB(sqlitecon.SQLiteConnectionManager(
            self.__database_dir + "/" + database_name))

class SQLiteTempDB(temp_db.TempDB):
    """SQLite implementation of TempDB."""
    def __init__(self, connection_manager):
        """Create an SQLiteTempDB object.
        Args:
            connection_manager: and SQLiteConnectionManager object
        """
        self.__mgr = connection_manager
        with self.__mgr.get_connection() as conn:
            conn.execute("CREATE TABLE IF NOT EXISTS entries("
                         "id INTEGER PRIMARY KEY, entry BLOB)")
        self.__tables = ["entries"]

    def __repr__(self):
        return "%r(%r)" % (self.__class__.__name__, self.__mgr)

    def __str__(self):
        return "%s(%s, tables: %s): " % (self.__class__.__name__,
                                         self.__mgr, self.__tables)

    # Not part of the abstract interface: used to identify the database file
    # we're writing to.
    def db_name(self):
        return self.__mgr.db_name

    def drop_entries(self):
        """Drop all entries."""
        with self.__mgr.get_connection() as conn:
            conn.execute("DELETE FROM entries")

    def store_entries(self, entries):
        """Batch store log entries.
        Args:
            entries: an iterable of (entry_number, client_pb2.EntryResponse)
                     tuples
        """
        with self.__mgr.get_connection() as conn:
            cursor = conn.cursor()
            serialized_entries = map(lambda x: (
                    x[0], sqlite3.Binary(x[1].SerializeToString())), entries)
            try:
                cursor.executemany("INSERT OR REPLACE INTO entries(id, entry) VALUES "
                                   "(?, ?)", serialized_entries)
            except sqlite3.IntegrityError as e:
                raise database.KeyError("Failed to insert entries: an entry "
                                        "with the given sequence number "
                                        "already exists\n%s" % e)

    def scan_entries(self, start, end):
        """Retrieve log entries.
        Args:
            start: index of the first entry to retrieve.
            end: index of the last entry to retrieve.
        Yields:
            client_pb2.EntryResponse protos
        Raises:
            KeyError: an entry with a sequence number in the range does not
                      exist.
        """
        with self.__mgr.get_connection() as conn:
            cursor = conn.cursor()
            next_id = start
            for row in cursor.execute("SELECT id, entry FROM entries WHERE id "
                                      "BETWEEN ? and ? ORDER BY id ASC",
                                      (start, end)):
                if row["id"] != next_id:
                    raise database.KeyError("No such entry: %d" % next_id)
                entry = client_pb2.EntryResponse()
                entry.ParseFromString(str(row["entry"]))
                yield entry
                next_id += 1
            if next_id != end + 1:
                raise database.KeyError("No such entry: %d" % next_id)



import hashlib, base64, sys

from verifiable_log import VerifiableLog

# Making this shorter affects the Merkle tree size only, can be helpful for debugging.
SHA_LEN = 256

# Convenience
def hashit(x):
  return hashlib.sha256(x).digest()

def hashleaf(x):
  return hashit(chr(0) + x)

def hashparent(a, b):
  return hashit(chr(1) + a + b)

# Pre-compute defaults for empty leaves
DEFAULTS = [hashleaf('')]
for i in range(SHA_LEN):
  DEFAULTS.append(hashparent(DEFAULTS[-1], DEFAULTS[-1]))
DEFAULTS = DEFAULTS[::-1]


# Internal utility class for keep a "version-tracked" value
class SequencedData:
  def __init__(self, default=None):
    self._seqs = [0]
    self._vals = [default]

  # Get the value that was in effect at seq time.
  def get(self, seq):
    i = len(self._seqs) - 1  #TODO replace with bin search
    while self._seqs[i] > seq and i > 0:
      i -= 1
    return self._vals[i]

  # Set a new value in effect as of seq time.  seq must be greater than current max.
  def set(self, value, seq):
    if self.get(seq) == value:
      return False
    #elif seq == self._seqs[-1]:
    #  self._vals[-1] = value
    #  return True
    elif seq > self._seqs[-1]:
      self._seqs.append(seq)
      self._vals.append(value)
      return True
    else:
      raise

  # For debug
  def debug_dump(self):
    for s, v in zip(self._seqs, self._vals):
      print 's', repr(s), 'v', repr(v)

# Represent node in sparse merkle tree
class Node:
  def __init__(self, parent=None):
    # The parent - never changes, only None for root
    self._parent = parent

    # The depth - never changes
    self._depth = parent._depth + 1 if parent else 0

    # Hash for this node - changed by _redo_hash()
    self._hash = SequencedData(DEFAULTS[self._depth])

    # The left child node - initially None, then set, then unchanged.
    self._left = SequencedData()

    # The right child node - initially None, then set, then unchanged.
    self._right = SequencedData()

    # If _value is set, the contains the full path to that value.
    # As an optimization we store a value a high in the tree as is currently
    # unique.  We need this path to (a) calculate inclusion proofs correctly for
    # distant cousins and (b) so that we can push the value lower as more values
    # are added
    self._path = SequencedData()

    # The value for a node.  This should never be set if a left or right child
    # node is present.
    self._value = SequencedData()

    # The following is informational only...
    self._node_count = 1
    while parent:
      parent._node_count += 1
      parent = parent._parent

  # Return the hash of the left subtree, even if None
  def left_hash(self, seq):
    x = self._left.get(seq)
    return x.hash(seq) if x else DEFAULTS[self._depth + 1]

  # Return the hash of the left subtree, even if None
  def right_hash(self, seq):
    x = self._right.get(seq)
    return x.hash(seq) if x else DEFAULTS[self._depth + 1]

  def left(self, seq):
    return self._left.get(seq)

  def right(self, seq):
    return self._right.get(seq)

  def value(self, seq):
    return self._value.get(seq)

  def path(self, seq):
    return self._path.get(seq)

  def set_left(self, left, seq):
    self._left.set(left, seq)

  def set_right(self, right, seq):
    self._right.set(right, seq)

  # Change the value - will always recalc hash of all ancestors
  def set_value(self, value, seq):
    self._value.set(value, seq)
    self._redo_hash(seq)

  def set_path(self, path, seq):
    self._path.set(path, seq)

  # Recalc our hash, and all ancestors.  Even though we cheat and optimize
  # the storage of the tree into minimal nodes, we still hash 256 levels deep.
  def _redo_hash(self, seq):
    v = self._value.get(seq)
    if v is None:
      if self._left.get(seq) is None and self._right.get(seq) is None:
        h = DEFAULTS[self._depth]
      else:
        h = hashparent(self.left_hash(seq), self.right_hash(seq))
    else:
      h = hashleaf(v)
      p = self._path.get(seq)
      for i in range(SHA_LEN, self._depth, -1):
        if p[i - 1]:
          h = hashparent(DEFAULTS[i], h)
        else:
          h = hashparent(h, DEFAULTS[i])

    if self._hash.set(h, seq):
      if self._parent is not None:
        self._parent._redo_hash(seq)

  def hash(self, seq):
    return self._hash.get(seq)

  # Dump all info out for debugging
  def debug_dump(self, seq):
    print ('  ' * self._depth) + "H: " + base64.b64encode(self.hash(seq))
    v = self._value.get(seq)
    if v is None:
      x = self.left(seq)
      if x:
        print ('  ' * self._depth) + "L:"
        x.debug_dump(seq)
      else:
        print ('  ' * self._depth) + "L: " + base64.b64encode(self.left_hash(seq))
      x = self.right(seq)
      if x:
        print ('  ' * self._depth) + "R:"
        x.debug_dump(seq)
      else:
        print ('  ' * self._depth) + "R: " + base64.b64encode(self.right_hash(seq))
    else:
        print ('  ' * self._depth) + "V:" + repr(v)
        print ('  ' * self._depth) + "P:" + ''.join([str(int(x)) for x in self.path(seq)])

# Take a key as string and produce 256 boolean values indicating left (False) or right (True)
def construct_key_path(key):
  return [x == '1' for x in ''.join(('%8s' % bin(ord(x))[2:]) for x in hashlib.sha256(key).digest())[:SHA_LEN]]

# Take a proof array and replace any default values (ie empty trees) with the
# count of how many consecutive there are.  Finally base64 encode each other one.
# e.g. [xxxx, yyyyy, zzzzz, empty1, empty2, empty255] would convert to [xxx, yyy, zzz, 253]
def compress_proof(proof):
  counter = 0
  rv = []
  for y in ['' if x == DEFAULTS[idx + 1] else base64.b64encode(x) for idx, x in enumerate(proof)]:
    if y == '':
      counter += 1
    else:
      if counter > 0:
        rv.append(counter)
        counter = 0
      rv.append(y)
  if counter > 0:
    rv.append(counter)
  return rv

# Undo what compress_proof does.  Output is 256 raw byte arrays.
def decompress_proof(proof):
  pp = []
  for y in proof:
    if type(y) == int:
      pp += [''] * y
    else:
      pp.append(y)
  return [DEFAULTS[idx + 1] if x == '' else base64.b64decode(x) for idx, x in enumerate(pp)]

# Map that knows nothing about logs.
class VerifiableMap:
  def __init__(self):
    self._root = Node()

    # Every actual mutation bumps this by one.
    self._tree_size = 0

  # Internal method called recursively by put initially
  # cur is the current node, idx is what level inside of path we are at,
  # value is the value we want to place.
  def _place(self, cur, idx, path, value):
    if idx < SHA_LEN:
      vs = [value]
      ps = [path]

      prev_value = cur.value(self._tree_size)
      if prev_value is not None:
        prev_path = cur.path(self._tree_size)
        if prev_path != path:
          vs.append(prev_value)
          ps.append(prev_path)

        self._tree_size += 1
        cur.set_path(None, self._tree_size)

        self._tree_size += 1
        cur.set_value(None, self._tree_size)

      for v, p in zip(vs, ps):
        l = cur.left(self._tree_size)
        r = cur.right(self._tree_size)

        if l is None and r is None and len(vs) == 1:
          self._place(cur, SHA_LEN, p, v)
        else:
          if p[idx]: # right
            next = r
            if next is None:
              next = Node(cur)
              self._tree_size += 1
              cur.set_right(next, self._tree_size)
          else: # left
            next = l
            if next is None:
              next = Node(cur)
              self._tree_size += 1
              cur.set_left(next, self._tree_size)
          self._place(next, idx + 1, p, v)
    else:
      self._tree_size += 1
      cur.set_path(path, self._tree_size)

      self._tree_size += 1
      cur.set_value(value, self._tree_size)

  # Set key to this value. Will likely cause multiple tree mutations.
  def put(self, key, value):
    self._place(self._root, 0, construct_key_path(key), value)

  # Return the latest value for a key without any proof
  # Should only be used by friendly classes.
  def get_latest(self, key):
    cur = self._root
    path = construct_key_path(key)
    for ch in path:
      if cur is None or cur.path(self._tree_size) == path:
        break
      if ch: # right
        cur = cur.right(self._tree_size)
      else: # left
        cur = cur.left(self._tree_size)
    return cur.value(self._tree_size) if cur else ''

  # For a given key and seq return the value as of that time and a proof
  # proof is "compressed" and already base64 encoded.
  # Will not mutate the tree
  def get(self, key, seq):
    last = cur = self._root
    proof = []
    path = construct_key_path(key)
    for ch in path:
      if cur is None or cur.path(seq) == path:
        break
      else:
        last = cur
        if cur.value(seq) is not None:
          cur = None
        else:
          if ch: # right
            proof.append(cur.left_hash(seq))
            cur = cur.right(seq)
          else: # left
            proof.append(cur.right_hash(seq))
            cur = cur.left(seq)

    if len(proof) < SHA_LEN:
      v = last.value(seq)
      if v is not None:
        p = last.path(seq)
        while len(proof) < SHA_LEN and path[len(proof)] == p[len(proof)]:
          proof.append(DEFAULTS[len(proof) + 1])
        if len(proof) < SHA_LEN:
          h = hashleaf(v)
          for i in range(SHA_LEN, len(proof) + 1, -1):
            if p[i - 1]:
              h = hashparent(DEFAULTS[i], h)
            else:
              h = hashparent(h, DEFAULTS[i])
          proof.append(h)

    if len(proof) < SHA_LEN:
      proof += DEFAULTS[-(SHA_LEN - len(proof)):]

    return cur.value(seq) if cur else '', compress_proof(proof)

  # Return the tree size (# of mutations) and the root hash as of that time.
  def get_tree_head(self, seq=None):
    if seq is None:
      seq = self._tree_size
    rv = {
      'tree_size': seq,
      'sha256_root_hash': base64.b64encode(self._root.hash(seq)),
    }
    return rv

# Utility method for checking an inclusion proof
def recalc_tree_hash(key, value, proof):
  t = hashleaf(value)
  for ch, p in zip(construct_key_path(key), decompress_proof(proof))[::-1]:
    if ch:
      t = hashparent(p, t)
    else:
      t = hashparent(t, p)
  return base64.b64encode(t)
